@@ -7,16 +7,12 @@
  * Full text of license available in license.txt file, in main folder
  *
  */
-#include "CFocusableHelper.h"
-#include "../Global.h"
 #include "StdInc.h"
-#include "widgets/TextControls.h"
+#include "CFocusableHelper.h"
+#include "widgets/CTextInput.h"
 
 void removeFocusFromActiveInput()
 {
-    if(CFocusable::inputWithFocus == nullptr)
-        return;
-    CFocusable::inputWithFocus->focus = false;
-    CFocusable::inputWithFocus->redraw();
-    CFocusable::inputWithFocus = nullptr;
+    if(CFocusable::inputWithFocus != nullptr)
+        CFocusable::inputWithFocus->removeFocus();
 }
