@@ -38,7 +38,6 @@
 #include "../../lib/CConfigHandler.h"
 #include "../../lib/CGeneralTextHandler.h"
 #include "../../lib/CHeroHandler.h"
-#include "../../lib/CondSh.h"
 #include "../../lib/gameState/InfoAboutArmy.h"
 #include "../../lib/mapObjects/CGTownInstance.h"
 #include "../../lib/networkPacks/PacksForClientBattle.h"
@@ -58,6 +57,7 @@ BattleInterface::BattleInterface(const BattleID & battleID, const CCreatureSet *
 	, curInt(att)
 	, battleID(battleID)
 	, battleOpeningDelayActive(true)
+	, round(0)
 {
 	if(spectatorInt)
 	{
@@ -94,7 +94,7 @@ BattleInterface::BattleInterface(const BattleID & battleID, const CCreatureSet *
 	obstacleController.reset(new BattleObstacleController(*this));
 
 	adventureInt->onAudioPaused();
-	ongoingAnimationsState.set(true);
+	ongoingAnimationsState.setBusy();
 
 	GH.windows().pushWindow(windowObject);
 	windowObject->blockUI(true);
@@ -107,7 +107,8 @@ void BattleInterface::playIntroSoundAndUnlockInterface()
 {
 	auto onIntroPlayed = [this]()
 	{
-		if(LOCPLINT->battleInt)
+		// Make sure that battle have not ended while intro was playing AND that a different one has not started
+		if(LOCPLINT->battleInt.get() == this)
 			onIntroSoundPlayed();
 	};
 
@@ -125,7 +126,7 @@ void BattleInterface::playIntroSoundAndUnlockInterface()
 	}
 }
 
-bool BattleInterface::openingPlaying()
+bool BattleInterface::openingPlaying() const
 {
 	return battleOpeningDelayActive;
 }
@@ -234,6 +235,7 @@ void BattleInterface::newRoundFirst()
 void BattleInterface::newRound()
 {
 	console->addText(CGI->generaltexth->allTexts[412]);
+	round++;
 }
 
 void BattleInterface::giveCommand(EActionType action, BattleHex tile, SpellID spell)
@@ -330,7 +332,7 @@ void BattleInterface::battleFinished(const BattleResult& br, QueryID queryID)
 	GH.windows().pushWindow(wnd);
 
 	curInt->waitWhileDialog(); // Avoid freeze when AI end turn after battle. Check bug #1897
-	CPlayerInterface::battleInt = nullptr;
+	CPlayerInterface::battleInt.reset();
 }
 
 void BattleInterface::spellCast(const BattleSpellCast * sc)
@@ -349,13 +351,13 @@ void BattleInterface::spellCast(const BattleSpellCast * sc)
 	CCS->curh->set(Cursor::Combat::BLOCKED);
 
 	const SpellID spellID = sc->spellID;
+
+	if(!spellID.hasValue())
+		return;
+
 	const CSpell * spell = spellID.toSpell();
 	auto targetedTile = sc->tile;
 
-	assert(spell);
-	if(!spell)
-		return;
-
 	const AudioPath & castSoundPath = spell->getCastSound();
 
 	if (!castSoundPath.empty())
@@ -637,7 +639,7 @@ void BattleInterface::tacticPhaseEnd()
 
 static bool immobile(const CStack *s)
 {
-	return !s->speed(0, true); //should bound stacks be immobile?
+	return s->getMovementRange() == 0; //should bound stacks be immobile?
 }
 
 void BattleInterface::tacticNextStack(const CStack * current)
@@ -726,7 +728,7 @@ void BattleInterface::requestAutofightingAIToTakeAction()
 			// FIXME: unsafe
 			// Run task in separate thread to avoid UI lock while AI is making turn (which might take some time)
 			// HOWEVER this thread won't atttempt to lock game state, potentially leading to races
-			boost::thread aiThread([this, activeStack]()
+			boost::thread aiThread([battleID = this->battleID, curInt = this->curInt, activeStack]()
 			{
 				setThreadName("autofightingAI");
 				curInt->autofightingAI->activeStack(battleID, activeStack);
@@ -741,6 +743,11 @@ void BattleInterface::castThisSpell(SpellID spellID)
 	actionsController->castThisSpell(spellID);
 }
 
+void BattleInterface::endNetwork()
+{
+	ongoingAnimationsState.requestTermination();
+}
+
 void BattleInterface::executeStagedAnimations()
 {
 	EAnimationEvents earliestStage = EAnimationEvents::COUNT;
@@ -772,19 +779,19 @@ void BattleInterface::executeAnimationStage(EAnimationEvents event)
 
 void BattleInterface::onAnimationsStarted()
 {
-	ongoingAnimationsState.setn(true);
+	ongoingAnimationsState.setBusy();
 }
 
 void BattleInterface::onAnimationsFinished()
 {
-	ongoingAnimationsState.setn(false);
+	ongoingAnimationsState.setFree();
 }
 
 void BattleInterface::waitForAnimations()
 {
 	{
 		auto unlockInterface = vstd::makeUnlockGuard(GH.interfaceMutex);
-		ongoingAnimationsState.waitUntil(false);
+		ongoingAnimationsState.waitWhileBusy();
 	}
 
 	assert(!hasAnimations());
@@ -799,7 +806,7 @@ void BattleInterface::waitForAnimations()
 
 bool BattleInterface::hasAnimations()
 {
-	return ongoingAnimationsState.get();
+	return ongoingAnimationsState.isBusy();
 }
 
 void BattleInterface::checkForAnimations()
