@@ -15,6 +15,7 @@ struct SDL_KeyboardEvent;
 /// Class that handles keyboard input from SDL events
 class InputSourceKeyboard
 {
+	std::string getKeyNameWithModifiers(const std::string & keyName) const;
 public:
 	InputSourceKeyboard();
 
@@ -22,6 +23,7 @@ public:
 	void handleEventKeyUp(const SDL_KeyboardEvent & current);
 
 	bool isKeyboardAltDown() const;
+	bool isKeyboardCmdDown() const;
 	bool isKeyboardCtrlDown() const;
 	bool isKeyboardShiftDown() const;
 };
