@@ -10,7 +10,6 @@
 #pragma once
 
 VCMI_LIB_NAMESPACE_BEGIN
-template <typename T> struct CondSh;
 class Point;
 class Rect;
 VCMI_LIB_NAMESPACE_END
@@ -62,9 +61,17 @@ public:
 	/// May not match size of window if user has UI scaling different from 100%
 	Point screenDimensions() const;
 
-	/// returns true if chosen keyboard key is currently pressed down
+	/// returns true if Alt is currently pressed down
 	bool isKeyboardAltDown() const;
+	/// returns true if Ctrl is currently pressed down
+	/// on Apple system, this also tests for Cmd key
+	/// For use with keyboard-based events
 	bool isKeyboardCtrlDown() const;
+	/// on Apple systems, returns true if Cmd key is pressed
+	/// on other systems, returns true if Ctrl is pressed
+	/// /// For use with mouse-based events
+	bool isKeyboardCmdDown() const;
+	/// returns true if Shift is currently pressed down
 	bool isKeyboardShiftDown() const;
 
 	void startTextInput(const Rect & where);
@@ -92,8 +99,8 @@ public:
 	void init();
 	void renderFrame();
 
-	/// called whenever user selects different resolution, requiring to center/resize all windows
-	void onScreenResize();
+	/// called whenever SDL_WINDOWEVENT_RESTORED is reported or the user selects a different resolution, requiring to center/resize all windows
+	void onScreenResize(bool resolutionChanged);
 
 	void handleEvents(); //takes events from queue and calls interested objects
 	void fakeMouseMove();
