@@ -24,8 +24,12 @@
 #include "../gui/Shortcut.h"
 #include "../gui/WindowHandler.h"
 #include "../render/Canvas.h"
+#include "../globalLobby/GlobalLobbyLoginWindow.h"
+#include "../globalLobby/GlobalLobbyClient.h"
+#include "../globalLobby/GlobalLobbyWindow.h"
 #include "../widgets/CComponent.h"
 #include "../widgets/Buttons.h"
+#include "../widgets/CTextInput.h"
 #include "../widgets/MiscWidgets.h"
 #include "../widgets/ObjectLists.h"
 #include "../widgets/TextControls.h"
@@ -42,9 +46,7 @@
 #include "../../CCallback.h"
 
 #include "../../lib/CGeneralTextHandler.h"
-#include "../../lib/JsonNode.h"
 #include "../../lib/campaign/CampaignHandler.h"
-#include "../../lib/serializer/Connection.h"
 #include "../../lib/serializer/CTypeList.h"
 #include "../../lib/filesystem/Filesystem.h"
 #include "../../lib/filesystem/CCompressedStream.h"
@@ -56,12 +58,6 @@
 #include "../../lib/CConfigHandler.h"
 #include "../../lib/GameConstants.h"
 #include "../../lib/CRandomGenerator.h"
-#include "../../lib/CondSh.h"
-
-#if defined(SINGLE_PROCESS_APP) && defined(VCMI_ANDROID)
-#include "../../server/CVCMIServer.h"
-#include <SDL.h>
-#endif
 
 std::shared_ptr<CMainMenu> CMM;
 ISelectionScreenInfo * SEL;
@@ -112,9 +108,9 @@ void CMenuScreen::show(Canvas & to)
 	if(!config["video"].isNull())
 	{
 		// redraw order: background -> video -> buttons and pictures
-		background->redraw();
+		background->showAll(to);
 		CCS->videoh->update((int)config["video"]["x"].Float() + pos.x, (int)config["video"]["y"].Float() + pos.y, to.getInternalSurface(), true, false);
-		tabs->redraw();
+		tabs->showAll(to);
 	}
 	CIntObject::show(to);
 }
@@ -185,11 +181,11 @@ static std::function<void()> genCommand(CMenuScreen * menu, std::vector<std::str
 				switch(std::find(gameType.begin(), gameType.end(), commands.front()) - gameType.begin())
 				{
 				case 0:
-					return std::bind(CMainMenu::openLobby, ESelectionScreen::newGame, true, nullptr, ELoadMode::NONE);
+					return []() { CMainMenu::openLobby(ESelectionScreen::newGame, true, {}, ELoadMode::NONE);};
 				case 1:
 					return []() { GH.windows().createAndPushWindow<CMultiMode>(ESelectionScreen::newGame); };
 				case 2:
-					return std::bind(CMainMenu::openLobby, ESelectionScreen::campaignList, true, nullptr, ELoadMode::NONE);
+					return []() { CMainMenu::openLobby(ESelectionScreen::campaignList, true, {}, ELoadMode::NONE);};
 				case 3:
 					return std::bind(CMainMenu::startTutorial);
 				}
@@ -200,13 +196,14 @@ static std::function<void()> genCommand(CMenuScreen * menu, std::vector<std::str
 				switch(std::find(gameType.begin(), gameType.end(), commands.front()) - gameType.begin())
 				{
 				case 0:
-					return std::bind(CMainMenu::openLobby, ESelectionScreen::loadGame, true, nullptr, ELoadMode::SINGLE);
+					return []() { CMainMenu::openLobby(ESelectionScreen::loadGame, true, {}, ELoadMode::SINGLE);};
 				case 1:
 					return []() { GH.windows().createAndPushWindow<CMultiMode>(ESelectionScreen::loadGame); };
 				case 2:
-					return std::bind(CMainMenu::openLobby, ESelectionScreen::loadGame, true, nullptr, ELoadMode::CAMPAIGN);
+					return []() { CMainMenu::openLobby(ESelectionScreen::loadGame, true, {}, ELoadMode::CAMPAIGN);};
 				case 3:
-					return std::bind(CMainMenu::openLobby, ESelectionScreen::loadGame, true, nullptr, ELoadMode::TUTORIAL);
+					return []() { CMainMenu::openLobby(ESelectionScreen::loadGame, true, {}, ELoadMode::TUTORIAL);};
+
 				}
 			}
 			break;
@@ -244,6 +241,11 @@ std::shared_ptr<CButton> CMenuEntry::createButton(CMenuScreen * parent, const Js
 
 	EShortcut shortcut = GH.shortcuts().findShortcut(button["shortcut"].String());
 
+	if (shortcut == EShortcut::NONE && !button["shortcut"].String().empty())
+	{
+		logGlobal->warn("Unknown shortcut '%s' found when loading main menu config!", button["shortcut"].String());
+	}
+
 	auto result = std::make_shared<CButton>(Point(posx, posy), AnimationPath::fromJson(button["name"]), help, command, shortcut);
 
 	if (button["center"].Bool())
@@ -263,7 +265,7 @@ CMenuEntry::CMenuEntry(CMenuScreen * parent, const JsonNode & config)
 	for(const JsonNode & node : config["buttons"].Vector())
 	{
 		buttons.push_back(createButton(parent, node));
-		buttons.back()->hoverable = true;
+		buttons.back()->setHoverable(true);
 		buttons.back()->setRedrawParent(true);
 	}
 }
@@ -272,12 +274,13 @@ CMainMenuConfig::CMainMenuConfig()
 	: campaignSets(JsonPath::builtin("config/campaignSets.json"))
 	, config(JsonPath::builtin("config/mainmenu.json"))
 {
-
+	if (config["game-select"].Vector().empty())
+		handleFatalError("Main menu config is invalid or corrupted. Please disable any mods or reinstall VCMI", false);
 }
 
-CMainMenuConfig & CMainMenuConfig::get()
+const CMainMenuConfig & CMainMenuConfig::get()
 {
-	static CMainMenuConfig config;
+	static const CMainMenuConfig config;
 	return config;
 }
 
@@ -356,10 +359,9 @@ void CMainMenu::update()
 	GH.windows().simpleRedraw();
 }
 
-void CMainMenu::openLobby(ESelectionScreen screenType, bool host, const std::vector<std::string> * names, ELoadMode loadMode)
+void CMainMenu::openLobby(ESelectionScreen screenType, bool host, const std::vector<std::string> & names, ELoadMode loadMode)
 {
-	CSH->resetStateForLobby(screenType == ESelectionScreen::newGame ? StartInfo::NEW_GAME : StartInfo::LOAD_GAME, names);
-	CSH->screenType = screenType;
+	CSH->resetStateForLobby(screenType == ESelectionScreen::newGame ? EStartMode::NEW_GAME : EStartMode::LOAD_GAME, screenType, EServerMode::LOCAL, names);
 	CSH->loadMode = loadMode;
 
 	GH.windows().createAndPushWindow<CSimpleJoinScreen>(host);
@@ -374,20 +376,36 @@ void CMainMenu::openCampaignLobby(const std::string & campaignFileName, std::str
 
 void CMainMenu::openCampaignLobby(std::shared_ptr<CampaignState> campaign)
 {
-	CSH->resetStateForLobby(StartInfo::CAMPAIGN);
-	CSH->screenType = ESelectionScreen::campaignList;
+	CSH->resetStateForLobby(EStartMode::CAMPAIGN, ESelectionScreen::campaignList, EServerMode::LOCAL, {});
 	CSH->campaignStateToSend = campaign;
 	GH.windows().createAndPushWindow<CSimpleJoinScreen>();
 }
 
 void CMainMenu::openCampaignScreen(std::string name)
 {
-	if(vstd::contains(CMainMenuConfig::get().getCampaigns().Struct(), name))
+	auto const & config = CMainMenuConfig::get().getCampaigns();
+
+	if(!vstd::contains(config.Struct(), name))
 	{
-		GH.windows().createAndPushWindow<CCampaignScreen>(CMainMenuConfig::get().getCampaigns(), name);
+		logGlobal->error("Unknown campaign set: %s", name);
 		return;
 	}
-	logGlobal->error("Unknown campaign set: %s", name);
+
+	bool campaignsFound = true;
+	for (auto const & entry : config[name]["items"].Vector())
+	{
+		ResourcePath resourceID(entry["file"].String(), EResType::CAMPAIGN);
+		if (!CResourceHandler::get()->existsResource(resourceID))
+			campaignsFound = false;
+	}
+
+	if (!campaignsFound)
+	{
+		CInfoWindow::showInfoDialog(CGI->generaltexth->translate("vcmi.client.errors.missingCampaigns"), std::vector<std::shared_ptr<CComponent>>(), PlayerColor(1));
+		return;
+	}
+
+	GH.windows().createAndPushWindow<CCampaignScreen>(config, name);
 }
 
 void CMainMenu::startTutorial()
@@ -401,7 +419,7 @@ void CMainMenu::startTutorial()
 		
 	auto mapInfo = std::make_shared<CMapInfo>();
 	mapInfo->mapInit(tutorialMap.getName());
-	CMainMenu::openLobby(ESelectionScreen::newGame, true, nullptr, ELoadMode::NONE);
+	CMainMenu::openLobby(ESelectionScreen::newGame, true, {}, ELoadMode::NONE);
 	CSH->startMapAfterConnection(mapInfo);
 }
 
@@ -437,14 +455,23 @@ CMultiMode::CMultiMode(ESelectionScreen ScreenType)
 	statusBar = CGStatusBar::create(std::make_shared<CPicture>(background->getSurface(), Rect(7, 465, 440, 18), 7, 465));
 	playerName = std::make_shared<CTextInput>(Rect(19, 436, 334, 16), background->getSurface());
 	playerName->setText(getPlayerName());
-	playerName->cb += std::bind(&CMultiMode::onNameChange, this, _1);
+	playerName->setCallback(std::bind(&CMultiMode::onNameChange, this, _1));
+
+	buttonHotseat = std::make_shared<CButton>(Point(373, 78 + 57 * 0), AnimationPath::builtin("MUBHOT.DEF"), CGI->generaltexth->zelp[266], std::bind(&CMultiMode::hostTCP, this), EShortcut::MAIN_MENU_HOTSEAT);
+	buttonLobby = std::make_shared<CButton>(Point(373, 78 + 57 * 1), AnimationPath::builtin("MUBONL.DEF"), CGI->generaltexth->zelp[265], std::bind(&CMultiMode::openLobby, this), EShortcut::MAIN_MENU_LOBBY);
+
+	buttonHost = std::make_shared<CButton>(Point(373, 78 + 57 * 3), AnimationPath::builtin("MUBHOST.DEF"), CButton::tooltip(CGI->generaltexth->translate("vcmi.mainMenu.hostTCP"), ""), std::bind(&CMultiMode::hostTCP, this), EShortcut::MAIN_MENU_HOST_GAME);
+	buttonJoin = std::make_shared<CButton>(Point(373, 78 + 57 * 4), AnimationPath::builtin("MUBJOIN.DEF"), CButton::tooltip(CGI->generaltexth->translate("vcmi.mainMenu.joinTCP"), ""), std::bind(&CMultiMode::joinTCP, this), EShortcut::MAIN_MENU_JOIN_GAME);
 
-	buttonHotseat = std::make_shared<CButton>(Point(373, 78), AnimationPath::builtin("MUBHOT.DEF"), CGI->generaltexth->zelp[266], std::bind(&CMultiMode::hostTCP, this));
-	buttonHost = std::make_shared<CButton>(Point(373, 78 + 57 * 1), AnimationPath::builtin("MUBHOST.DEF"), CButton::tooltip(CGI->generaltexth->translate("vcmi.mainMenu.hostTCP"), ""), std::bind(&CMultiMode::hostTCP, this));
-	buttonJoin = std::make_shared<CButton>(Point(373, 78 + 57 * 2), AnimationPath::builtin("MUBJOIN.DEF"), CButton::tooltip(CGI->generaltexth->translate("vcmi.mainMenu.joinTCP"), ""), std::bind(&CMultiMode::joinTCP, this));
 	buttonCancel = std::make_shared<CButton>(Point(373, 424), AnimationPath::builtin("MUBCANC.DEF"), CGI->generaltexth->zelp[288], [=](){ close();}, EShortcut::GLOBAL_CANCEL);
 }
 
+void CMultiMode::openLobby()
+{
+	close();
+	CSH->getGlobalLobby().activateInterface();
+}
+
 void CMultiMode::hostTCP()
 {
 	auto savedScreenType = screenType;
@@ -463,7 +490,7 @@ std::string CMultiMode::getPlayerName()
 {
 	std::string name = settings["general"]["playerName"].String();
 	if(name == "Player")
-		name = CGI->generaltexth->translate("vcmi.mainMenu.playerName");
+		name = CGI->generaltexth->translate("core.genrltxt.434");
 	return name;
 }
 
@@ -487,15 +514,15 @@ CMultiPlayers::CMultiPlayers(const std::string & firstPlayer, ESelectionScreen S
 	for(int i = 0; i < inputNames.size(); i++)
 	{
 		inputNames[i] = std::make_shared<CTextInput>(Rect(60, 85 + i * 30, 280, 16), background->getSurface());
-		inputNames[i]->cb += std::bind(&CMultiPlayers::onChange, this, _1);
+		inputNames[i]->setCallback(std::bind(&CMultiPlayers::onChange, this, _1));
 	}
 
 	buttonOk = std::make_shared<CButton>(Point(95, 338), AnimationPath::builtin("MUBCHCK.DEF"), CGI->generaltexth->zelp[560], std::bind(&CMultiPlayers::enterSelectionScreen, this), EShortcut::GLOBAL_ACCEPT);
 	buttonCancel = std::make_shared<CButton>(Point(205, 338), AnimationPath::builtin("MUBCANC.DEF"), CGI->generaltexth->zelp[561], [=](){ close();}, EShortcut::GLOBAL_CANCEL);
 	statusBar = CGStatusBar::create(std::make_shared<CPicture>(background->getSurface(), Rect(7, 381, 348, 18), 7, 381));
 
-	inputNames[0]->setText(firstPlayer, true);
-#ifndef VCMI_IOS
+	inputNames[0]->setText(firstPlayer);
+#ifndef VCMI_MOBILE
 	inputNames[0]->giveFocus();
 #endif
 }
@@ -516,7 +543,7 @@ void CMultiPlayers::enterSelectionScreen()
 	Settings name = settings.write["general"]["playerName"];
 	name->String() = names[0];
 
-	CMainMenu::openLobby(screenType, host, &names, loadMode);
+	CMainMenu::openLobby(screenType, host, names, loadMode);
 }
 
 CSimpleJoinScreen::CSimpleJoinScreen(bool host)
@@ -533,18 +560,19 @@ CSimpleJoinScreen::CSimpleJoinScreen(bool host)
 	{
 		textTitle->setText(CGI->generaltexth->translate("vcmi.mainMenu.serverConnecting"));
 		buttonOk->block(true);
-		startConnectThread();
+		startConnection();
 	}
 	else
 	{
 		textTitle->setText(CGI->generaltexth->translate("vcmi.mainMenu.serverAddressEnter"));
-		inputAddress->cb += std::bind(&CSimpleJoinScreen::onChange, this, _1);
-		inputPort->cb += std::bind(&CSimpleJoinScreen::onChange, this, _1);
-		inputPort->filters += std::bind(&CTextInput::numberFilter, _1, _2, 0, 65535);
+		inputAddress->setCallback(std::bind(&CSimpleJoinScreen::onChange, this, _1));
+		inputPort->setCallback(std::bind(&CSimpleJoinScreen::onChange, this, _1));
+		inputPort->setFilterNumber(0, 65535);
 		inputAddress->giveFocus();
 	}
-	inputAddress->setText(host ? CServerHandler::localhostAddress : CSH->getHostAddress(), true);
-	inputPort->setText(std::to_string(CSH->getHostPort()), true);
+	inputAddress->setText(host ? CSH->getLocalHostname() : CSH->getRemoteHostname());
+	inputPort->setText(std::to_string(host ? CSH->getLocalPort() : CSH->getRemotePort()));
+	buttonOk->block(inputAddress->getText().empty() || inputPort->getText().empty());
 
 	buttonCancel = std::make_shared<CButton>(Point(142, 142), AnimationPath::builtin("MUBCANC.DEF"), CGI->generaltexth->zelp[561], std::bind(&CSimpleJoinScreen::leaveScreen, this), EShortcut::GLOBAL_CANCEL);
 	statusBar = CGStatusBar::create(std::make_shared<CPicture>(background->getSurface(), Rect(7, 186, 218, 18), 7, 186));
@@ -556,20 +584,13 @@ void CSimpleJoinScreen::connectToServer()
 	buttonOk->block(true);
 	GH.stopTextInput();
 
-	startConnectThread(inputAddress->getText(), boost::lexical_cast<ui16>(inputPort->getText()));
+	startConnection(inputAddress->getText(), boost::lexical_cast<ui16>(inputPort->getText()));
 }
 
 void CSimpleJoinScreen::leaveScreen()
 {
-	if(CSH->state == EClientState::CONNECTING)
-	{
-		textTitle->setText(CGI->generaltexth->translate("vcmi.mainMenu.serverClosing"));
-		CSH->state = EClientState::CONNECTION_CANCELLED;
-	}
-	else if(GH.windows().isTopWindow(this))
-	{
-		close();
-	}
+	textTitle->setText(CGI->generaltexth->translate("vcmi.mainMenu.serverClosing"));
+	CSH->setState(EClientState::CONNECTION_CANCELLED);
 }
 
 void CSimpleJoinScreen::onChange(const std::string & newText)
@@ -577,43 +598,12 @@ void CSimpleJoinScreen::onChange(const std::string & newText)
 	buttonOk->block(inputAddress->getText().empty() || inputPort->getText().empty());
 }
 
-void CSimpleJoinScreen::startConnectThread(const std::string & addr, ui16 port)
+void CSimpleJoinScreen::startConnection(const std::string & addr, ui16 port)
 {
-#if defined(SINGLE_PROCESS_APP) && defined(VCMI_ANDROID)
-	// in single process build server must use same JNIEnv as client
-	// as server runs in a separate thread, it must not attempt to search for Java classes (and they're already cached anyway)
-	// https://github.com/libsdl-org/SDL/blob/main/docs/README-android.md#threads-and-the-java-vm
-	CVCMIServer::reuseClientJNIEnv(SDL_AndroidGetJNIEnv());
-#endif
-	boost::thread connector(&CSimpleJoinScreen::connectThread, this, addr, port);
-
-	connector.detach();
-}
-
-void CSimpleJoinScreen::connectThread(const std::string & addr, ui16 port)
-{
-	setThreadName("connectThread");
-	if(!addr.length())
-		CSH->startLocalServerAndConnect();
+	if(addr.empty())
+		CSH->startLocalServerAndConnect(false);
 	else
-		CSH->justConnectToServer(addr, port);
-
-	// async call to prevent thread race
-	GH.dispatchMainThread([this](){
-		if(CSH->state == EClientState::CONNECTION_FAILED)
-		{
-			CInfoWindow::showInfoDialog(CGI->generaltexth->translate("vcmi.mainMenu.serverConnectionFailed"), {});
-
-			textTitle->setText(CGI->generaltexth->translate("vcmi.mainMenu.serverAddressEnter"));
-			GH.startTextInput(inputAddress->pos);
-			buttonOk->block(false);
-		}
-
-		if(GH.windows().isTopWindow(this))
-		{
-			close();
-		}
-	});
+		CSH->connectToServer(addr, port);
 }
 
 CLoadingScreen::CLoadingScreen()
@@ -628,7 +618,8 @@ CLoadingScreen::CLoadingScreen()
 	const auto & conf = CMainMenuConfig::get().getConfig()["loading"];
 	if(conf.isStruct())
 	{
-		const int posx = conf["x"].Integer(), posy = conf["y"].Integer();
+		const int posx = conf["x"].Integer();
+		const int posy = conf["y"].Integer();
 		const int blockSize = conf["size"].Integer();
 		const int blocksAmount = conf["amount"].Integer();
 		
