@@ -16,6 +16,7 @@
 #include "MapViewCache.h"
 #include "MapViewModel.h"
 
+#include "../CCallback.h"
 #include "../CPlayerInterface.h"
 #include "../adventureMap/AdventureMapInterface.h"
 #include "../gui/CGuiHandler.h"
@@ -23,6 +24,8 @@
 #include "../eventsSDL/InputHandler.h"
 
 #include "../../lib/CConfigHandler.h"
+#include "../../lib/StartInfo.h"
+#include "../../lib/UnlockGuard.h"
 #include "../../lib/mapObjects/CGHeroInstance.h"
 #include "../../lib/mapObjects/MiscObjects.h"
 #include "../../lib/pathfinder/CGPathNode.h"
@@ -86,7 +89,7 @@ void MapViewController::setTileSize(const Point & tileSize)
 	setViewCenter(newViewCenter, model->getLevel());
 }
 
-void MapViewController::modifyTileSize(int stepsChange)
+void MapViewController::modifyTileSize(int stepsChange, bool useDeadZone)
 {
 	// we want to zoom in/out in fixed 10% steps, to allow player to return back to exactly 100% zoom just by scrolling
 	// so, zooming in for 5 steps will put game at 1.1^5 = 1.61 scale
@@ -115,10 +118,13 @@ void MapViewController::modifyTileSize(int stepsChange)
 	if (actualZoom != currentZoom)
 	{
 		targetTileSize = actualZoom;
-		if(actualZoom.x >= defaultTileSize - zoomTileDeadArea && actualZoom.x <= defaultTileSize + zoomTileDeadArea)
-			actualZoom.x = defaultTileSize;
-		if(actualZoom.y >= defaultTileSize - zoomTileDeadArea && actualZoom.y <= defaultTileSize + zoomTileDeadArea)
-			actualZoom.y = defaultTileSize;
+		if (useDeadZone)
+		{
+			if(actualZoom.x >= defaultTileSize - zoomTileDeadArea && actualZoom.x <= defaultTileSize + zoomTileDeadArea)
+				actualZoom.x = defaultTileSize;
+			if(actualZoom.y >= defaultTileSize - zoomTileDeadArea && actualZoom.y <= defaultTileSize + zoomTileDeadArea)
+				actualZoom.y = defaultTileSize;
+		}
 		
 		bool isInDeadZone = targetTileSize != actualZoom || actualZoom == Point(defaultTileSize, defaultTileSize);
 
@@ -269,9 +275,15 @@ void MapViewController::afterRender()
 
 bool MapViewController::isEventInstant(const CGObjectInstance * obj, const PlayerColor & initiator)
 {
+	if(settings["gameTweaks"]["skipAdventureMapAnimations"].Bool())
+		return true;
+
 	if (!isEventVisible(obj, initiator))
 		return true;
 
+	if (!initiator.isValidPlayer())
+		return true; // skip effects such as new monsters on new month
+
 	if(initiator != LOCPLINT->playerID && settings["adventure"]["enemyMoveTime"].Float() <= 0)
 		return true; // instant movement speed
 
@@ -292,9 +304,14 @@ bool MapViewController::isEventVisible(const CGObjectInstance * obj, const Playe
 	if(!GH.windows().isTopWindow(adventureInt))
 		return false;
 
-	// do not focus on actions of other players during our turn (e.g. simturns)
-	if (LOCPLINT->makingTurn && initiator != LOCPLINT->playerID)
-		return false;
+	// do not focus on actions of other players except for AI with simturns off
+	if (initiator != LOCPLINT->playerID && initiator.isValidPlayer())
+	{
+		if (LOCPLINT->makingTurn)
+			return false;
+		if (LOCPLINT->cb->getStartInfo()->playerInfos.at(initiator).isControlledByHuman() && !settings["session"]["adventureTrackHero"].Bool())
+			return false;
+	}
 
 	if(obj->isVisitable())
 		return context->isVisible(obj->visitablePos());
@@ -313,9 +330,14 @@ bool MapViewController::isEventVisible(const CGHeroInstance * obj, const int3 &
 	if(!GH.windows().isTopWindow(adventureInt))
 		return false;
 
-	// do not focus on actions of other players during our turn (e.g. simturns)
-	if (LOCPLINT->makingTurn && obj->getOwner() != LOCPLINT->playerID)
-		return false;
+	// do not focus on actions of other players except for AI with simturns off
+	if (obj->getOwner() != LOCPLINT->playerID)
+	{
+		if (LOCPLINT->makingTurn)
+			return false;
+		if (LOCPLINT->cb->getStartInfo()->playerInfos.at(obj->getOwner()).isControlledByHuman() && !settings["session"]["adventureTrackHero"].Bool())
+			return false;
+	}
 
 	if(context->isVisible(obj->convertToVisitablePos(from)))
 		return true;
@@ -328,6 +350,7 @@ bool MapViewController::isEventVisible(const CGHeroInstance * obj, const int3 &
 
 void MapViewController::fadeOutObject(const CGObjectInstance * obj)
 {
+	animationWait.setBusy();
 	logGlobal->debug("Starting fade out animation");
 	fadingOutContext = std::make_shared<MapRendererAdventureFadingContext>(*state);
 	fadingOutContext->animationTime = adventureContext->animationTime;
@@ -348,6 +371,7 @@ void MapViewController::fadeOutObject(const CGObjectInstance * obj)
 
 void MapViewController::fadeInObject(const CGObjectInstance * obj)
 {
+	animationWait.setBusy();
 	logGlobal->debug("Starting fade in animation");
 	fadingInContext = std::make_shared<MapRendererAdventureFadingContext>(*state);
 	fadingInContext->animationTime = adventureContext->animationTime;
@@ -487,6 +511,7 @@ void MapViewController::onAfterHeroTeleported(const CGHeroInstance * obj, const
 
 	if(isEventVisible(obj, from, dest))
 	{
+		animationWait.setBusy();
 		logGlobal->debug("Starting teleport animation");
 		teleportContext = std::make_shared<MapRendererAdventureTransitionContext>(*state);
 		teleportContext->animationTime = adventureContext->animationTime;
@@ -522,6 +547,7 @@ void MapViewController::onHeroMoved(const CGHeroInstance * obj, const int3 & fro
 
 	if(movementTime > 1)
 	{
+		animationWait.setBusy();
 		logGlobal->debug("Starting movement animation");
 		movementContext = std::make_shared<MapRendererAdventureMovingContext>(*state);
 		movementContext->animationTime = adventureContext->animationTime;
@@ -559,6 +585,17 @@ bool MapViewController::hasOngoingAnimations()
 	return false;
 }
 
+void MapViewController::waitForOngoingAnimations()
+{
+	auto unlockInterface = vstd::makeUnlockGuard(GH.interfaceMutex);
+	animationWait.waitWhileBusy();
+}
+
+void MapViewController::endNetwork()
+{
+	animationWait.requestTermination();
+}
+
 void MapViewController::activateAdventureContext(uint32_t animationTime)
 {
 	resetContext();
@@ -624,6 +661,7 @@ void MapViewController::resetContext()
 	worldViewContext.reset();
 	spellViewContext.reset();
 	puzzleMapContext.reset();
+	animationWait.setFree();
 }
 
 void MapViewController::setTerrainVisibility(bool showAllTerrain)
