@@ -15,7 +15,6 @@
 
 #include "../CPlayerInterface.h"
 #include "../gui/CGuiHandler.h"
-#include "../gui/Shortcut.h"
 #include "../windows/CMessage.h"
 #include "../windows/InfoWindows.h"
 #include "../adventureMap/CInGameConsole.h"
@@ -30,9 +29,6 @@
 #include "lib/CAndroidVMHelper.h"
 #endif
 
-std::list<CFocusable*> CFocusable::focusables;
-CFocusable * CFocusable::inputWithFocus;
-
 std::string CLabel::visibleText()
 {
 	return text;
@@ -47,8 +43,8 @@ void CLabel::showAll(Canvas & to)
 
 }
 
-CLabel::CLabel(int x, int y, EFonts Font, ETextAlignment Align, const ColorRGBA & Color, const std::string & Text)
-	: CTextContainer(Align, Font, Color), text(Text)
+CLabel::CLabel(int x, int y, EFonts Font, ETextAlignment Align, const ColorRGBA & Color, const std::string & Text, int maxWidth)
+	: CTextContainer(Align, Font, Color), text(Text), maxWidth(maxWidth)
 {
 	setRedrawParent(true);
 	autoRedraw = true;
@@ -56,6 +52,8 @@ CLabel::CLabel(int x, int y, EFonts Font, ETextAlignment Align, const ColorRGBA
 	pos.y += y;
 	pos.w = pos.h = 0;
 
+	trimText();
+
 	if(alignment == ETextAlignment::TOPLEFT) // causes issues for MIDDLE
 	{
 		pos.w = (int)graphics->fonts[font]->getStringWidth(visibleText().c_str());
@@ -80,7 +78,12 @@ void CLabel::setAutoRedraw(bool value)
 
 void CLabel::setText(const std::string & Txt)
 {
+	assert(TextOperations::isValidUnicodeString(Txt));
+
 	text = Txt;
+	
+	trimText();
+
 	if(autoRedraw)
 	{
 		if(background || !parent)
@@ -90,6 +93,31 @@ void CLabel::setText(const std::string & Txt)
 	}
 }
 
+void CLabel::clear()
+{
+	text.clear();
+
+	if(autoRedraw)
+	{
+		if(background || !parent)
+			redraw();
+		else
+			parent->redraw();
+	}
+}
+
+void CLabel::setMaxWidth(int width)
+{
+	maxWidth = width;
+}
+
+void CLabel::trimText()
+{
+	if(maxWidth > 0)
+		while ((int)graphics->fonts[font]->getStringWidth(visibleText().c_str()) > maxWidth)
+			TextOperations::trimRightUnicode(text);
+}
+
 void CLabel::setColor(const ColorRGBA & Color)
 {
 	color = Color;
@@ -104,7 +132,7 @@ void CLabel::setColor(const ColorRGBA & Color)
 
 size_t CLabel::getWidth()
 {
-	return graphics->fonts[font]->getStringWidth(visibleText());;
+	return graphics->fonts[font]->getStringWidth(visibleText());
 }
 
 CMultiLineLabel::CMultiLineLabel(Rect position, EFonts Font, ETextAlignment Align, const ColorRGBA & Color, const std::string & Text) :
@@ -166,29 +194,24 @@ void CTextContainer::blitLine(Canvas & to, Rect destRect, std::string what)
 
 	// input is rect in which given text should be placed
 	// calculate proper position for top-left corner of the text
-	if(alignment == ETextAlignment::TOPLEFT)
-	{
+
+	if(alignment == ETextAlignment::TOPLEFT || alignment == ETextAlignment::CENTERLEFT  || alignment == ETextAlignment::BOTTOMLEFT)
 		where.x += getBorderSize().x;
+
+	if(alignment == ETextAlignment::CENTER || alignment == ETextAlignment::TOPCENTER || alignment == ETextAlignment::BOTTOMCENTER)
+		where.x += (destRect.w - (static_cast<int>(f->getStringWidth(what)) - delimitersCount)) / 2;
+
+	if(alignment == ETextAlignment::TOPRIGHT || alignment == ETextAlignment::BOTTOMRIGHT || alignment == ETextAlignment::CENTERRIGHT)
+		where.x += getBorderSize().x + destRect.w - (static_cast<int>(f->getStringWidth(what)) - delimitersCount);
+
+	if(alignment == ETextAlignment::TOPLEFT || alignment == ETextAlignment::TOPCENTER || alignment == ETextAlignment::TOPRIGHT)
 		where.y += getBorderSize().y;
-	}
 
-	if(alignment == ETextAlignment::TOPCENTER)
-	{
-		where.x += (int(destRect.w) - int(f->getStringWidth(what) - delimitersCount)) / 2;
-		where.y += getBorderSize().y;
-	}
+	if(alignment == ETextAlignment::CENTERLEFT || alignment == ETextAlignment::CENTER || alignment == ETextAlignment::CENTERRIGHT)
+		where.y += (destRect.h - static_cast<int>(f->getLineHeight())) / 2;
 
-	if(alignment == ETextAlignment::CENTER)
-	{
-		where.x += (int(destRect.w) - int(f->getStringWidth(what) - delimitersCount)) / 2;
-		where.y += (int(destRect.h) - int(f->getLineHeight())) / 2;
-	}
-
-	if(alignment == ETextAlignment::BOTTOMRIGHT)
-	{
-		where.x += getBorderSize().x + destRect.w - ((int)f->getStringWidth(what) - delimitersCount);
-		where.y += getBorderSize().y + destRect.h - (int)f->getLineHeight();
-	}
+	if(alignment == ETextAlignment::BOTTOMLEFT || alignment == ETextAlignment::BOTTOMCENTER || alignment == ETextAlignment::BOTTOMRIGHT)
+		where.y += getBorderSize().y + destRect.h - static_cast<int>(f->getLineHeight());
 
 	size_t begin = 0;
 	size_t currDelimeter = 0;
@@ -352,6 +375,17 @@ void CTextBox::sliderMoved(int to)
 	label->scrollTextTo(to);
 }
 
+void CTextBox::trimToFit()
+{
+	if (slider)
+		return;
+
+	pos.w = label->textSize.x;
+	pos.h = label->textSize.y;
+	label->pos.w = label->textSize.x;
+	label->pos.h = label->textSize.y;
+}
+
 void CTextBox::resize(Point newSize)
 {
 	pos.w = newSize.x;
@@ -371,11 +405,12 @@ void CTextBox::setText(const std::string & text)
 	{
 		// slider is no longer needed
 		slider.reset();
+		label->scrollTextTo(0);
 	}
 	else if(slider)
 	{
 		// decrease width again if slider still used
-		label->pos.w = pos.w - 32;
+		label->pos.w = pos.w - 16;
 		assert(label->pos.w > 0);
 		label->setText(text);
 		slider->setAmount(label->textSize.y);
@@ -383,12 +418,12 @@ void CTextBox::setText(const std::string & text)
 	else if(label->textSize.y > label->pos.h)
 	{
 		// create slider and update widget
-		label->pos.w = pos.w - 32;
+		label->pos.w = pos.w - 16;
 		assert(label->pos.w > 0);
 		label->setText(text);
 
 		OBJECT_CONSTRUCTION_CUSTOM_CAPTURING(255 - DISPOSE);
-		slider = std::make_shared<CSlider>(Point(pos.w - 32, 0), pos.h, std::bind(&CTextBox::sliderMoved, this, _1),
+		slider = std::make_shared<CSlider>(Point(pos.w - 16, 0), pos.h, std::bind(&CTextBox::sliderMoved, this, _1),
 			label->pos.h, label->textSize.y, 0, Orientation::VERTICAL, CSlider::EStyle(sliderStyle));
 		slider->setScrollStep((int)graphics->fonts[label->font]->getLineHeight());
 		slider->setPanningStep(1);
@@ -444,7 +479,7 @@ void CGStatusBar::clear()
 }
 
 CGStatusBar::CGStatusBar(std::shared_ptr<CIntObject> background_, EFonts Font, ETextAlignment Align, const ColorRGBA & Color)
-	: CLabel(background_->pos.x, background_->pos.y, Font, Align, Color, "")
+	: CLabel(background_->pos.x, background_->pos.y, Font, Align, Color, "", background_->pos.w)
 	, enteringText(false)
 {
 	addUsedEvents(LCLICK);
@@ -533,290 +568,3 @@ Point CGStatusBar::getBorderSize()
 	assert(0);
 	return Point();
 }
-
-CTextInput::CTextInput(const Rect & Pos, EFonts font, const CFunctionList<void(const std::string &)> & CB, bool giveFocusToInput)
-	: CLabel(Pos.x, Pos.y, font, ETextAlignment::CENTER),
-	cb(CB),
-	CFocusable(std::make_shared<CKeyboardFocusListener>(this))
-{
-	setRedrawParent(true);
-	pos.h = Pos.h;
-	pos.w = Pos.w;
-	background.reset();
-	addUsedEvents(LCLICK | SHOW_POPUP | KEYBOARD | TEXTINPUT);
-
-#if !defined(VCMI_MOBILE)
-	if(giveFocusToInput)
-		giveFocus();
-#endif
-}
-
-CTextInput::CTextInput(const Rect & Pos, const Point & bgOffset, const ImagePath & bgName, const CFunctionList<void(const std::string &)> & CB)
-	:cb(CB), 	CFocusable(std::make_shared<CKeyboardFocusListener>(this))
-{
-	pos += Pos.topLeft();
-	pos.h = Pos.h;
-	pos.w = Pos.w;
-
-	OBJ_CONSTRUCTION;
-	background = std::make_shared<CPicture>(bgName, bgOffset.x, bgOffset.y);
-	addUsedEvents(LCLICK | SHOW_POPUP | KEYBOARD | TEXTINPUT);
-
-#if !defined(VCMI_MOBILE)
-	giveFocus();
-#endif
-}
-
-CTextInput::CTextInput(const Rect & Pos, std::shared_ptr<IImage> srf)
-	:CFocusable(std::make_shared<CKeyboardFocusListener>(this))
-{
-	pos += Pos.topLeft();
-	OBJ_CONSTRUCTION;
-	background = std::make_shared<CPicture>(srf, Pos);
-	pos.w = background->pos.w;
-	pos.h = background->pos.h;
-	background->pos = pos;
-	addUsedEvents(LCLICK | KEYBOARD | TEXTINPUT);
-
-#if !defined(VCMI_MOBILE)
-	giveFocus();
-#endif
-}
-
-std::atomic<int> CKeyboardFocusListener::usageIndex(0);
-
-CKeyboardFocusListener::CKeyboardFocusListener(CTextInput * textInput)
-	:textInput(textInput)
-{
-}
-
-void CKeyboardFocusListener::focusGot()
-{
-	GH.startTextInput(textInput->pos);
-	usageIndex++;
-}
-
-void CKeyboardFocusListener::focusLost()
-{
-	if(0 == --usageIndex)
-	{
-		GH.stopTextInput();
-	}
-}
-
-std::string CTextInput::visibleText()
-{
-	return focus ? text + newText + "_" : text;
-}
-
-void CTextInput::clickPressed(const Point & cursorPosition)
-{
-	if(!focus)
-		giveFocus();
-}
-
-void CTextInput::keyPressed(EShortcut key)
-{
-	if(!focus)
-		return;
-
-	if(key == EShortcut::GLOBAL_MOVE_FOCUS)
-	{
-		moveFocus();
-		return;
-	}
-
-	bool redrawNeeded = false;
-
-	switch(key)
-	{
-	case EShortcut::GLOBAL_BACKSPACE:
-		if(!newText.empty())
-		{
-			TextOperations::trimRightUnicode(newText);
-			redrawNeeded = true;
-		}
-		else if(!text.empty())
-		{
-			TextOperations::trimRightUnicode(text);
-			redrawNeeded = true;
-		}
-		break;
-	default:
-		break;
-	}
-
-	if(redrawNeeded)
-	{
-		redraw();
-		cb(text);
-	}
-}
-
-void CTextInput::showPopupWindow(const Point & cursorPosition)
-{
-	if(!helpBox.empty()) //there is no point to show window with nothing inside...
-		CRClickPopup::createAndPush(helpBox);
-}
-
-
-void CTextInput::setText(const std::string & nText)
-{
-	setText(nText, false);
-}
-
-void CTextInput::setText(const std::string & nText, bool callCb)
-{
-	CLabel::setText(nText);
-	if(callCb)
-		cb(text);
-}
-
-void CTextInput::setHelpText(const std::string & text)
-{
-	helpBox = text;
-}
-
-void CTextInput::textInputed(const std::string & enteredText)
-{
-	if(!focus)
-		return;
-	std::string oldText = text;
-
-	text += enteredText;
-
-	filters(text, oldText);
-	if(text != oldText)
-	{
-		redraw();
-		cb(text);
-	}
-	newText.clear();
-}
-
-void CTextInput::textEdited(const std::string & enteredText)
-{
-	if(!focus)
-		return;
-
-	newText = enteredText;
-	redraw();
-	cb(text + newText);
-}
-
-void CTextInput::filenameFilter(std::string & text, const std::string &)
-{
-	static const std::string forbiddenChars = "<>:\"/\\|?*\r\n"; //if we are entering a filename, some special characters won't be allowed
-	size_t pos;
-	while((pos = text.find_first_of(forbiddenChars)) != std::string::npos)
-		text.erase(pos, 1);
-}
-
-void CTextInput::numberFilter(std::string & text, const std::string & oldText, int minValue, int maxValue)
-{
-	assert(minValue < maxValue);
-
-	if(text.empty())
-		text = "0";
-
-	size_t pos = 0;
-	if(text[0] == '-') //allow '-' sign as first symbol only
-		pos++;
-
-	while(pos < text.size())
-	{
-		if(text[pos] < '0' || text[pos] > '9')
-		{
-			text = oldText;
-			return; //new text is not number.
-		}
-		pos++;
-	}
-	try
-	{
-		int value = boost::lexical_cast<int>(text);
-		if(value < minValue)
-			text = std::to_string(minValue);
-		else if(value > maxValue)
-			text = std::to_string(maxValue);
-	}
-	catch(boost::bad_lexical_cast &)
-	{
-		//Should never happen. Unless I missed some cases
-		logGlobal->warn("Warning: failed to convert %s to number!", text);
-		text = oldText;
-	}
-}
-
-CFocusable::CFocusable()
-	:CFocusable(std::make_shared<IFocusListener>())
-{
-}
-
-CFocusable::CFocusable(std::shared_ptr<IFocusListener> focusListener)
-	: focusListener(focusListener)
-{
-	focus = false;
-	focusables.push_back(this);
-}
-
-CFocusable::~CFocusable()
-{
-	if(hasFocus())
-	{
-		inputWithFocus = nullptr;
-		focusListener->focusLost();
-	}
-
-	focusables -= this;
-}
-
-bool CFocusable::hasFocus() const
-{
-	return inputWithFocus == this;
-}
-
-void CFocusable::giveFocus()
-{
-	focus = true;
-	focusListener->focusGot();
-	redraw();
-
-	if(inputWithFocus)
-	{
-		inputWithFocus->focus = false;
-		inputWithFocus->focusListener->focusLost();
-		inputWithFocus->redraw();
-	}
-
-	inputWithFocus = this;
-}
-
-void CFocusable::moveFocus()
-{
-	auto i = vstd::find(focusables, this),
-		ourIt = i;
-	for(i++; i != ourIt; i++)
-	{
-		if(i == focusables.end())
-			i = focusables.begin();
-
-		if((*i)->isActive())
-		{
-			(*i)->giveFocus();
-			break;
-		}
-	}
-}
-
-void CFocusable::removeFocus()
-{
-	if(this == inputWithFocus)
-	{
-		focus = false;
-		focusListener->focusLost();
-		redraw();
-
-		inputWithFocus = nullptr;
-	}
-}
