@@ -13,7 +13,6 @@
 #include "../gui/TextAlignment.h"
 #include "../render/Colors.h"
 #include "../render/EFont.h"
-#include "../../lib/FunctionList.h"
 #include "../../lib/filesystem/ResourcePath.h"
 
 class IImage;
@@ -43,9 +42,11 @@ class CLabel : public CTextContainer
 protected:
 	Point getBorderSize() override;
 	virtual std::string visibleText();
+	virtual void trimText();
 
 	std::shared_ptr<CIntObject> background;
 	std::string text;
+	int maxWidth;
 	bool autoRedraw;  //whether control will redraw itself on setTxt
 
 public:
@@ -53,11 +54,13 @@ public:
 	std::string getText();
 	virtual void setAutoRedraw(bool option);
 	virtual void setText(const std::string & Txt);
+	virtual void setMaxWidth(int width);
 	virtual void setColor(const ColorRGBA & Color);
+	void clear();
 	size_t getWidth();
 
 	CLabel(int x = 0, int y = 0, EFonts Font = FONT_SMALL, ETextAlignment Align = ETextAlignment::TOPLEFT,
-		const ColorRGBA & Color = Colors::WHITE, const std::string & Text = "");
+		const ColorRGBA & Color = Colors::WHITE, const std::string & Text = "", int maxWidth = 0);
 	void showAll(Canvas & to) override; //shows statusbar (with current text)
 };
 
@@ -113,6 +116,9 @@ public:
 	CTextBox(std::string Text, const Rect & rect, int SliderStyle, EFonts Font = FONT_SMALL, ETextAlignment Align = ETextAlignment::TOPLEFT, const ColorRGBA & Color = Colors::WHITE);
 
 	void resize(Point newSize);
+	/// Resizes text box to minimal size needed to fit current text
+	/// No effect if text is too large to fit and requires slider
+	void trimToFit();
 	void setText(const std::string & Txt);
 	void sliderMoved(int to);
 };
@@ -160,89 +166,4 @@ public:
 	void clear() override;
 	void setEnteringMode(bool on) override;
 	void setEnteredText(const std::string & text) override;
-
-};
-
-class CFocusable;
-
-class IFocusListener
-{
-public:
-	virtual void focusGot() {};
-	virtual void focusLost() {};
-	virtual ~IFocusListener() = default;
-};
-
-/// UIElement which can get input focus
-class CFocusable : public virtual CIntObject
-{
-private:
-	std::shared_ptr<IFocusListener> focusListener;
-
-public:
-	bool focus; //only one focusable control can have focus at one moment
-
-	void giveFocus(); //captures focus
-	void moveFocus(); //moves focus to next active control (may be used for tab switching)
-	void removeFocus(); //remove focus
-	bool hasFocus() const;
-
-	static std::list<CFocusable *> focusables; //all existing objs
-	static CFocusable * inputWithFocus; //who has focus now
-
-	CFocusable();
-	CFocusable(std::shared_ptr<IFocusListener> focusListener);
-	~CFocusable();
-};
-
-class CTextInput;
-class CKeyboardFocusListener : public IFocusListener
-{
-private:
-	static std::atomic<int> usageIndex;
-	CTextInput * textInput;
-
-public:
-	CKeyboardFocusListener(CTextInput * textInput);
-	void focusGot() override;
-	void focusLost() override;
-};
-
-/// Text input box where players can enter text
-class CTextInput : public CLabel, public CFocusable
-{
-	std::string newText;
-	std::string helpBox; //for right-click help
-	
-protected:
-	std::string visibleText() override;
-
-public:
-	
-	CFunctionList<void(const std::string &)> cb;
-	CFunctionList<void(std::string &, const std::string &)> filters;
-	void setText(const std::string & nText) override;
-	void setText(const std::string & nText, bool callCb);
-	void setHelpText(const std::string &);
-
-	CTextInput(const Rect & Pos, EFonts font, const CFunctionList<void(const std::string &)> & CB, bool giveFocusToInput = true);
-	CTextInput(const Rect & Pos, const Point & bgOffset, const ImagePath & bgName, const CFunctionList<void(const std::string &)> & CB);
-	CTextInput(const Rect & Pos, std::shared_ptr<IImage> srf);
-
-	void clickPressed(const Point & cursorPosition) override;
-	void keyPressed(EShortcut key) override;
-	void showPopupWindow(const Point & cursorPosition) override;
-
-	//bool captureThisKey(EShortcut key) override;
-
-	void textInputed(const std::string & enteredText) override;
-	void textEdited(const std::string & enteredText) override;
-
-	//Filter that will block all characters not allowed in filenames
-	static void filenameFilter(std::string & text, const std::string & oldText);
-	//Filter that will allow only input of numbers in range min-max (min-max are allowed)
-	//min-max should be set via something like std::bind
-	static void numberFilter(std::string & text, const std::string & oldText, int minValue, int maxValue);
-
-	friend class CKeyboardFocusListener;
 };
