@@ -16,6 +16,7 @@ class Services;
 
 class IGameInfoCallback;
 class IBattleInfoCallback;
+class BattleID;
 
 namespace events
 {
@@ -31,7 +32,7 @@ public:
 	virtual ~Environment() = default;
 
 	virtual const Services * services() const = 0;
-	virtual const BattleCb * battle() const = 0;
+	virtual const BattleCb * battle(const BattleID & battleID) const = 0;
 	virtual const GameCb * game() const = 0;
 	virtual vstd::CLoggerBase * logger() const = 0;
 	virtual events::EventBus * eventBus() const = 0;
