@@ -1,3 +1,12 @@
+/*
+ * StdInc.h, part of VCMI engine
+ *
+ * Authors: listed in file AUTHORS in main folder
+ *
+ * License: GNU General Public License v2.0 or later
+ * Full text of license available in license.txt file, in main folder
+ *
+ */
 #pragma once
 
 #include "../Global.h"
@@ -9,6 +18,7 @@
 #include <QList>
 #include <QString>
 #include <QFile>
+#include <QTemporaryDir> 
 
 VCMI_LIB_USING_NAMESPACE
 
