@@ -21,6 +21,7 @@
 
 #include "updatedialog_moc.h"
 #include "main.h"
+#include "helper.h"
 
 void MainWindow::load()
 {
@@ -45,7 +46,7 @@ void MainWindow::load()
 	QDir::addSearchPath("icons", pathToQString(VCMIDirs::get().userDataPath() / "launcher" / "icons"));
 #endif
 
-	settings.init("config/settings.json", "vcmi:settings");
+	Helper::loadSettings();
 }
 
 void MainWindow::computeSidePanelSizes()
@@ -53,7 +54,6 @@ void MainWindow::computeSidePanelSizes()
 	QVector<QToolButton*> widgets = {
 		ui->modslistButton,
 		ui->settingsButton,
-		ui->lobbyButton,
 		ui->aboutButton,
 		ui->startEditorButton,
 		ui->startGameButton
@@ -86,10 +86,6 @@ MainWindow::MainWindow(QWidget * parent)
 
 	ui->setupUi(this);
 	
-	connect(ui->lobbyView, &Lobby::enableMod, ui->modlistView, &CModListView::enableModByName);
-	connect(ui->lobbyView, &Lobby::disableMod, ui->modlistView, &CModListView::disableModByName);
-	connect(ui->modlistView, &CModListView::modsChanged, ui->lobbyView, &Lobby::updateMods);
-
 	//load window settings
 	QSettings s(Ui::teamName, Ui::appName);
 
@@ -136,14 +132,14 @@ void MainWindow::detectPreferredLanguage()
 		for (auto const & vcmiLang : Languages::getLanguageList())
 			if (vcmiLang.tagIETF == userLang.toStdString())
 				selectedLanguage = vcmiLang.identifier;
-	}
 
-	logGlobal->info("Selected language: %s", selectedLanguage);
-
-	if (!selectedLanguage.empty())
-	{
-		Settings node = settings.write["general"]["language"];
-		node->String() = selectedLanguage;
+		if (!selectedLanguage.empty())
+		{
+			logGlobal->info("Selected language: %s", selectedLanguage);
+			Settings node = settings.write["general"]["language"];
+			node->String() = selectedLanguage;
+			return;
+		}
 	}
 }
 
@@ -151,7 +147,6 @@ void MainWindow::enterSetup()
 {
 	ui->startGameButton->setEnabled(false);
 	ui->startEditorButton->setEnabled(false);
-	ui->lobbyButton->setEnabled(false);
 	ui->settingsButton->setEnabled(false);
 	ui->aboutButton->setEnabled(false);
 	ui->modslistButton->setEnabled(false);
@@ -165,7 +160,6 @@ void MainWindow::exitSetup()
 
 	ui->startGameButton->setEnabled(true);
 	ui->startEditorButton->setEnabled(true);
-	ui->lobbyButton->setEnabled(true);
 	ui->settingsButton->setEnabled(true);
 	ui->aboutButton->setEnabled(true);
 	ui->modslistButton->setEnabled(true);
@@ -228,12 +222,6 @@ void MainWindow::on_settingsButton_clicked()
 	ui->tabListWidget->setCurrentIndex(TabRows::SETTINGS);
 }
 
-void MainWindow::on_lobbyButton_clicked()
-{
-	ui->startGameButton->setEnabled(false);
-	ui->tabListWidget->setCurrentIndex(TabRows::LOBBY);
-}
-
 void MainWindow::on_aboutButton_clicked()
 {
 	ui->startGameButton->setEnabled(true);
