@@ -9,18 +9,10 @@
  */
 #pragma once
 
-#include "StdInc.h"
-
 #include <QVariantMap>
 #include <QVariant>
 #include <QVector>
 
-VCMI_LIB_NAMESPACE_BEGIN
-
-class JsonNode;
-
-VCMI_LIB_NAMESPACE_END
-
 namespace ModStatus
 {
 enum EModStatus
@@ -62,8 +54,10 @@ public:
 	bool isCompatible() const;
 	// returns true if mod should be visible in Launcher
 	bool isVisible() const;
-	// installed and enabled
+	// returns true if mod type is Translation
 	bool isTranslation() const;
+	// returns true if mod type is Compatibility
+	bool isCompatibilityPatch() const;
 	// returns true if this is a submod
 	bool isSubmod() const;
 
