@@ -51,15 +51,19 @@ class CModListView : public QWidget
 	// find mods that depend on this one
 	QStringList findDependentMods(QString mod, bool excludeDisabled);
 
+	void manualInstallFile(QUrl url);
 	void downloadFile(QString file, QString url, QString description, qint64 size = 0);
 
 	void installMods(QStringList archives);
+	void installMaps(QStringList maps);
 	void installFiles(QStringList mods);
 
 	QString genChangelogText(CModEntry & mod);
 	QString genModInfoText(CModEntry & mod);
 
 	void changeEvent(QEvent *event) override;
+	void dragEnterEvent(QDragEnterEvent* event) override;
+	void dropEvent(QDropEvent *event) override;
 signals:
 	void modsChanged();
 
@@ -98,6 +102,7 @@ private slots:
 	void dataChanged(const QModelIndex & topleft, const QModelIndex & bottomRight);
 	void modSelected(const QModelIndex & current, const QModelIndex & previous);
 	void downloadProgress(qint64 current, qint64 max);
+	void extractionProgress(qint64 current, qint64 max);
 	void downloadFinished(QStringList savedFiles, QStringList failedFiles, QStringList errors);
 	void modelReset();
 	void hideProgressBar();
@@ -116,6 +121,8 @@ private slots:
 
 	void on_installButton_clicked();
 
+	void on_installFromFileButton_clicked();
+
 	void on_pushButton_clicked();
 
 	void on_refreshButton_clicked();
