@@ -24,7 +24,9 @@ namespace
 {
 QString detectModArchive(QString path, QString modName, std::vector<std::string> & filesToExtract)
 {
-	filesToExtract = ZipArchive::listFiles(qstringToPath(path));
+	ZipArchive archive(qstringToPath(path));
+
+	filesToExtract = archive.listFiles();
 
 	QString modDirName;
 
@@ -155,13 +157,10 @@ bool CModManager::canInstallMod(QString modname)
 	auto mod = modList->getMod(modname);
 
 	if(mod.isSubmod())
-		return addError(modname, "Can not install submod");
+		return addError(modname, tr("Can not install submod"));
 
 	if(mod.isInstalled())
-		return addError(modname, "Mod is already installed");
-
-	if(!mod.isAvailable())
-		return addError(modname, "Mod is not available");
+		return addError(modname, tr("Mod is already installed"));
 	return true;
 }
 
@@ -170,10 +169,10 @@ bool CModManager::canUninstallMod(QString modname)
 	auto mod = modList->getMod(modname);
 
 	if(mod.isSubmod())
-		return addError(modname, "Can not uninstall submod");
+		return addError(modname, tr("Can not uninstall submod"));
 
 	if(!mod.isInstalled())
-		return addError(modname, "Mod is not installed");
+		return addError(modname, tr("Mod is not installed"));
 
 	return true;
 }
@@ -183,21 +182,24 @@ bool CModManager::canEnableMod(QString modname)
 	auto mod = modList->getMod(modname);
 
 	if(mod.isEnabled())
-		return addError(modname, "Mod is already enabled");
+		return addError(modname, tr("Mod is already enabled"));
 
 	if(!mod.isInstalled())
-		return addError(modname, "Mod must be installed first");
+		return addError(modname, tr("Mod must be installed first"));
 
 	//check for compatibility
 	if(!mod.isCompatible())
-		return addError(modname, "Mod is not compatible, please update VCMI and checkout latest mod revisions");
+		return addError(modname, tr("Mod is not compatible, please update VCMI and checkout latest mod revisions"));
 
 	for(auto modEntry : mod.getDependencies())
 	{
 		if(!modList->hasMod(modEntry)) // required mod is not available
-			return addError(modname, QString("Required mod %1 is missing").arg(modEntry));
-		if(!modList->getMod(modEntry).isEnabled())
-			return addError(modname, QString("Required mod %1 is not enabled").arg(modEntry));
+			return addError(modname, tr("Required mod %1 is missing").arg(modEntry));
+
+		CModEntry modData = modList->getMod(modEntry);
+
+		if(!modData.isCompatibilityPatch() && !modData.isEnabled())
+			return addError(modname, tr("Required mod %1 is not enabled").arg(modEntry));
 	}
 
 	for(QString modEntry : modList->getModList())
@@ -206,14 +208,14 @@ bool CModManager::canEnableMod(QString modname)
 
 		// "reverse conflict" - enabled mod has this one as conflict
 		if(mod.isEnabled() && mod.getConflicts().contains(modname))
-			return addError(modname, QString("This mod conflicts with %1").arg(modEntry));
+			return addError(modname, tr("This mod conflicts with %1").arg(modEntry));
 	}
 
 	for(auto modEntry : mod.getConflicts())
 	{
 		// check if conflicting mod installed and enabled
 		if(modList->hasMod(modEntry) && modList->getMod(modEntry).isEnabled())
-			return addError(modname, QString("This mod conflicts with %1").arg(modEntry));
+			return addError(modname, tr("This mod conflicts with %1").arg(modEntry));
 	}
 	return true;
 }
@@ -223,17 +225,17 @@ bool CModManager::canDisableMod(QString modname)
 	auto mod = modList->getMod(modname);
 
 	if(mod.isDisabled())
-		return addError(modname, "Mod is already disabled");
+		return addError(modname, tr("Mod is already disabled"));
 
 	if(!mod.isInstalled())
-		return addError(modname, "Mod must be installed first");
+		return addError(modname, tr("Mod must be installed first"));
 
 	for(QString modEntry : modList->getModList())
 	{
 		auto current = modList->getMod(modEntry);
 
 		if(current.getDependencies().contains(modname) && current.isEnabled())
-			return addError(modname, QString("This mod is needed to run %1").arg(modEntry));
+			return addError(modname, tr("This mod is needed to run %1").arg(modEntry));
 	}
 	return true;
 }
@@ -272,34 +274,44 @@ bool CModManager::doEnableMod(QString mod, bool on)
 
 bool CModManager::doInstallMod(QString modname, QString archivePath)
 {
-	QString destDir = CLauncherDirs::get().modsPath() + "/";
+	const auto destDir = CLauncherDirs::modsPath() + QChar{'/'};
 
 	if(!QFile(archivePath).exists())
-		return addError(modname, "Mod archive is missing");
+		return addError(modname, tr("Mod archive is missing"));
 
 	if(localMods.contains(modname))
-		return addError(modname, "Mod with such name is already installed");
+		return addError(modname, tr("Mod with such name is already installed"));
 
 	std::vector<std::string> filesToExtract;
 	QString modDirName = ::detectModArchive(archivePath, modname, filesToExtract);
 	if(!modDirName.size())
-		return addError(modname, "Mod archive is invalid or corrupted");
+		return addError(modname, tr("Mod archive is invalid or corrupted"));
 	
-	auto futureExtract = std::async(std::launch::async, [&archivePath, &destDir, &filesToExtract]()
+	std::atomic<int> filesCounter = 0;
+
+	auto futureExtract = std::async(std::launch::async, [&archivePath, &destDir, &filesCounter, &filesToExtract]()
 	{
-		return ZipArchive::extract(qstringToPath(archivePath), qstringToPath(destDir), filesToExtract);
+		const auto destDirFsPath = qstringToPath(destDir);
+		ZipArchive archive(qstringToPath(archivePath));
+		for (auto const & file : filesToExtract)
+		{
+			if (!archive.extract(destDirFsPath, file))
+				return false;
+			++filesCounter;
+		}
+		return true;
 	});
 	
-	while(futureExtract.wait_for(std::chrono::milliseconds(50)) != std::future_status::ready)
+	while(futureExtract.wait_for(std::chrono::milliseconds(10)) != std::future_status::ready)
 	{
-		emit extractionProgress(0, 0);
+		emit extractionProgress(filesCounter, filesToExtract.size());
 		qApp->processEvents();
 	}
 	
 	if(!futureExtract.get())
 	{
 		removeModDir(destDir + modDirName);
-		return addError(modname, "Failed to extract mod data");
+		return addError(modname, tr("Failed to extract mod data"));
 	}
 
 	//rename folder and fix the path
@@ -327,11 +339,11 @@ bool CModManager::doUninstallMod(QString modname)
 	QString modDir = pathToQString(*CResourceHandler::get()->getResourceName(resID));
 
 	if(!QDir(modDir).exists())
-		return addError(modname, "Data with this mod was not found");
+		return addError(modname, tr("Data with this mod was not found"));
 
 	QDir modFullDir(modDir);
 	if(!removeModDir(modDir))
-		return addError(modname, "Mod is located in protected directory, please remove it manually:\n" + modFullDir.absolutePath());
+		return addError(modname, tr("Mod is located in protected directory, please remove it manually:\n") + modFullDir.absolutePath());
 
 	CResourceHandler::get("initial")->updateFilteredFiles([](const std::string &){ return true; });
 	loadMods();
