@@ -71,7 +71,7 @@ using TTeleportExitsList = std::vector<std::pair<ObjectInstanceID, int3>>;
 class DLL_LINKAGE CBattleGameInterface : public IBattleEventsReceiver
 {
 public:
-	bool human;
+	bool human = false;
 	PlayerColor playerID;
 	std::string dllName;
 
@@ -99,7 +99,7 @@ public:
 	// Show a dialog, player must take decision. If selection then he has to choose between one of given components,
 	// if cancel he is allowed to not choose. After making choice, CCallback::selectionMade should be called
 	// with number of selected component (1 - n) or 0 for cancel (if allowed) and askID.
-	virtual void showBlockingDialog(const std::string &text, const std::vector<Component> &components, QueryID askID, const int soundID, bool selection, bool cancel) = 0;
+	virtual void showBlockingDialog(const std::string &text, const std::vector<Component> &components, QueryID askID, const int soundID, bool selection, bool cancel, bool safeToAutoaccept) = 0;
 
 	// all stacks operations between these objects become allowed, interface has to call onEnd when done
 	virtual void showGarrisonDialog(const CArmedInstance *up, const CGHeroInstance *down, bool removableUnits, QueryID queryID) = 0;
@@ -111,8 +111,8 @@ public:
 
 	virtual std::optional<BattleAction> makeSurrenderRetreatDecision(const BattleID & battleID, const BattleStateInfoForRetreat & battleState) = 0;
 
-	virtual void saveGame(BinarySerializer & h, const int version) = 0;
-	virtual void loadGame(BinaryDeserializer & h, const int version) = 0;
+	virtual void saveGame(BinarySerializer & h) = 0;
+	virtual void loadGame(BinaryDeserializer & h) = 0;
 };
 
 class DLL_LINKAGE CDynLibHandler
@@ -144,26 +144,26 @@ public:
 	virtual std::string getBattleAIName() const = 0; //has to return name of the battle AI to be used
 
 	//battle interface
-	virtual void activeStack(const BattleID & battleID, const CStack * stack) override;
-	virtual void yourTacticPhase(const BattleID & battleID, int distance) override;
+	void activeStack(const BattleID & battleID, const CStack * stack) override;
+	void yourTacticPhase(const BattleID & battleID, int distance) override;
 
-	virtual void battleNewRound(const BattleID & battleID) override;
-	virtual void battleCatapultAttacked(const BattleID & battleID, const CatapultAttack & ca) override;
-	virtual void battleStart(const BattleID & battleID, const CCreatureSet *army1, const CCreatureSet *army2, int3 tile, const CGHeroInstance *hero1, const CGHeroInstance *hero2, bool side, bool replayAllowed) override;
-	virtual void battleStacksAttacked(const BattleID & battleID, const std::vector<BattleStackAttacked> & bsa, bool ranged) override;
-	virtual void actionStarted(const BattleID & battleID, const BattleAction &action) override;
-	virtual void battleNewRoundFirst(const BattleID & battleID) override;
-	virtual void actionFinished(const BattleID & battleID, const BattleAction &action) override;
-	virtual void battleStacksEffectsSet(const BattleID & battleID, const SetStackEffect & sse) override;
-	virtual void battleObstaclesChanged(const BattleID & battleID, const std::vector<ObstacleChanges> & obstacles) override;
-	virtual void battleStackMoved(const BattleID & battleID, const CStack * stack, std::vector<BattleHex> dest, int distance, bool teleport) override;
-	virtual void battleAttack(const BattleID & battleID, const BattleAttack *ba) override;
-	virtual void battleSpellCast(const BattleID & battleID, const BattleSpellCast *sc) override;
-	virtual void battleEnd(const BattleID & battleID, const BattleResult *br, QueryID queryID) override;
-	virtual void battleUnitsChanged(const BattleID & battleID, const std::vector<UnitChanges> & units) override;
+	void battleNewRound(const BattleID & battleID) override;
+	void battleCatapultAttacked(const BattleID & battleID, const CatapultAttack & ca) override;
+	void battleStart(const BattleID & battleID, const CCreatureSet *army1, const CCreatureSet *army2, int3 tile, const CGHeroInstance *hero1, const CGHeroInstance *hero2, bool side, bool replayAllowed) override;
+	void battleStacksAttacked(const BattleID & battleID, const std::vector<BattleStackAttacked> & bsa, bool ranged) override;
+	void actionStarted(const BattleID & battleID, const BattleAction &action) override;
+	void battleNewRoundFirst(const BattleID & battleID) override;
+	void actionFinished(const BattleID & battleID, const BattleAction &action) override;
+	void battleStacksEffectsSet(const BattleID & battleID, const SetStackEffect & sse) override;
+	void battleObstaclesChanged(const BattleID & battleID, const std::vector<ObstacleChanges> & obstacles) override;
+	void battleStackMoved(const BattleID & battleID, const CStack * stack, std::vector<BattleHex> dest, int distance, bool teleport) override;
+	void battleAttack(const BattleID & battleID, const BattleAttack *ba) override;
+	void battleSpellCast(const BattleID & battleID, const BattleSpellCast *sc) override;
+	void battleEnd(const BattleID & battleID, const BattleResult *br, QueryID queryID) override;
+	void battleUnitsChanged(const BattleID & battleID, const std::vector<UnitChanges> & units) override;
 
-	virtual void saveGame(BinarySerializer & h, const int version) override;
-	virtual void loadGame(BinaryDeserializer & h, const int version) override;
+	void saveGame(BinarySerializer & h) override;
+	void loadGame(BinaryDeserializer & h) override;
 };
 
 VCMI_LIB_NAMESPACE_END
