@@ -117,6 +117,8 @@ public:
 class DLL_LINKAGE TextLocalizationContainer
 {
 protected:
+	static std::recursive_mutex globalTextMutex;
+
 	struct StringState
 	{
 		/// Human-readable string that was added on registration
@@ -135,7 +137,7 @@ protected:
 		std::string modContext;
 		
 		template <typename Handler>
-		void serialize(Handler & h, const int Version)
+		void serialize(Handler & h)
 		{
 			h & baseValue;
 			h & baseLanguage;
@@ -153,6 +155,9 @@ protected:
 	
 	std::string getModLanguage(const std::string & modContext);
 	
+	// returns true if identifier with such name was registered, even if not translated to current language
+	bool identifierExists(const TextIdentifier & UID) const;
+
 public:
 	/// validates translation of specified language for specified mod
 	/// returns true if localization is valid and complete
@@ -163,9 +168,6 @@ public:
 	/// Any entries loaded by this will have priority over texts registered normally
 	void loadTranslationOverrides(const std::string & language, const std::string & modContext, JsonNode const & file);
 
-	// returns true if identifier with such name was registered, even if not translated to current language
-	bool identifierExists(const TextIdentifier & UID) const;
-	
 	/// add selected string to internal storage
 	void registerString(const std::string & modContext, const TextIdentifier & UID, const std::string & localized);
 	void registerString(const std::string & modContext, const TextIdentifier & UID, const std::string & localized, const std::string & language);
@@ -181,8 +183,9 @@ public:
 	/// converts identifier into user-readable string
 	const std::string & deserialize(const TextIdentifier & identifier) const;
 	
-	/// Debug method, dumps all currently known texts into console using Json-like format
-	void dumpAllTexts();
+	/// Debug method, returns all currently stored texts
+	/// Format: [mod ID][string ID] -> human-readable text
+	void exportAllTexts(std::map<std::string, std::map<std::string, std::string>> & storage) const;
 	
 	/// Add or override subcontainer which can store identifiers
 	void addSubContainer(const TextLocalizationContainer & container);
@@ -193,11 +196,24 @@ public:
 	void jsonSerialize(JsonNode & dest) const;
 	
 	template <typename Handler>
-	void serialize(Handler & h, const int Version)
+	void serialize(Handler & h)
 	{
+		std::lock_guard<std::recursive_mutex> globalLock(globalTextMutex);
+
 		std::string key;
 		auto sz = stringsLocalizations.size();
-		h & sz;
+
+		if (h.version >= Handler::Version::REMOVE_TEXT_CONTAINER_SIZE_T)
+		{
+			int64_t size = sz;
+			h & size;
+			sz = size;
+		}
+		else
+		{
+			h & sz;
+		}
+
 		if(h.saving)
 		{
 			for(auto s : stringsLocalizations)
@@ -218,6 +234,18 @@ public:
 	}
 };
 
+class DLL_LINKAGE TextContainerRegistrable : public TextLocalizationContainer
+{
+public:
+	TextContainerRegistrable();
+	~TextContainerRegistrable();
+
+	TextContainerRegistrable(const TextContainerRegistrable & other);
+	TextContainerRegistrable(TextContainerRegistrable && other) noexcept;
+
+	TextContainerRegistrable& operator=(const TextContainerRegistrable & b) = default;
+};
+
 /// Handles all text-related data in game
 class DLL_LINKAGE CGeneralTextHandler: public TextLocalizationContainer
 {
