@@ -16,17 +16,21 @@
 #include "CBonusTypeHandler.h"
 #include "BattleFieldHandler.h"
 #include "ObstacleHandler.h"
-#include "bonuses/CBonusSystemNode.h"
 #include "bonuses/Limiters.h"
 #include "bonuses/Propagators.h"
 #include "bonuses/Updaters.h"
 
+#include "networkPacks/ArtifactLocation.h"
 #include "serializer/CLoadFile.h"
 #include "serializer/CSaveFile.h"
 #include "rmg/CMapGenOptions.h"
 #include "mapObjectConstructors/AObjectTypeHandler.h"
 #include "mapObjectConstructors/CObjectClassesHandler.h"
+#include "mapObjects/CGMarket.h"
+#include "mapObjects/CGTownInstance.h"
 #include "mapObjects/CObjectHandler.h"
+#include "mapObjects/CQuest.h"
+#include "mapObjects/MiscObjects.h"
 #include "mapObjects/ObjectTemplate.h"
 #include "campaign/CampaignState.h"
 #include "StartInfo.h"
@@ -47,8 +51,6 @@
 #include "RiverHandler.h"
 #include "TerrainHandler.h"
 
-#include "serializer/Connection.h"
-
 VCMI_LIB_NAMESPACE_BEGIN
 
 void CPrivilegedInfoCallback::getFreeTiles(std::vector<int3> & tiles) const
@@ -266,6 +268,32 @@ CArmedInstance * CNonConstInfoCallback::getArmyInstance(const ObjectInstanceID &
 	return dynamic_cast<CArmedInstance *>(getObjInstance(oid));
 }
 
+CArtifactSet * CNonConstInfoCallback::getArtSet(const ArtifactLocation & loc)
+{
+	if(auto hero = getHero(loc.artHolder))
+	{
+		if(loc.creature.has_value())
+		{
+			if(loc.creature.value() == SlotID::COMMANDER_SLOT_PLACEHOLDER)
+				return hero->commander;
+			else
+				return hero->getStackPtr(loc.creature.value());
+		}
+		else
+		{
+			return hero;
+		}
+	}
+	else if(auto market = dynamic_cast<CGArtifactsAltar*>(getObjInstance(loc.artHolder)))
+	{
+		return market;
+	}
+	else
+	{
+		return nullptr;
+	}
+}
+
 bool IGameCallback::isVisitCoveredByAnotherQuery(const CGObjectInstance *obj, const CGHeroInstance *hero)
 {
 	//only server knows
