@@ -32,7 +32,7 @@ FactionID PlayerSettings::getCastleValidated() const
 {
 	if (!castle.isValid())
 		return FactionID(0);
-	if (castle.getNum() < VLC->townh->size() && VLC->townh->objects[castle.getNum()]->town != nullptr)
+	if (castle.getNum() < VLC->townh->size() && castle.toEntity(VLC)->hasTown())
 		return castle;
 
 	return FactionID(0);
@@ -89,15 +89,29 @@ std::string StartInfo::getCampaignName() const
 		return VLC->generaltexth->allTexts[508];
 }
 
+bool StartInfo::isSteadwickFallCampaignMission() const
+{
+	if (!campState)
+		return false;
+
+	if (campState->getFilename() != "DATA/EVIL1")
+		return false;
+
+	if (campState->currentScenario() != CampaignScenarioID(2))
+		return false;
+
+	return true;
+}
+
 void LobbyInfo::verifyStateBeforeStart(bool ignoreNoHuman) const
 {
 	if(!mi || !mi->mapHeader)
 		throw std::domain_error(VLC->generaltexth->translate("core.genrltxt.529"));
 	
 	auto missingMods = CMapService::verifyMapHeaderMods(*mi->mapHeader);
-	ModIncompatibility::ModListWithVersion modList;
+	ModIncompatibility::ModList modList;
 	for(const auto & m : missingMods)
-		modList.push_back({m.second.name, m.second.version.toString()});
+		modList.push_back(m.second.name);
 	
 	if(!modList.empty())
 		throw ModIncompatibility(modList);
@@ -111,7 +125,7 @@ void LobbyInfo::verifyStateBeforeStart(bool ignoreNoHuman) const
 	if(i == si->playerInfos.cend() && !ignoreNoHuman)
 		throw std::domain_error(VLC->generaltexth->translate("core.genrltxt.530"));
 
-	if(si->mapGenOptions && si->mode == StartInfo::NEW_GAME)
+	if(si->mapGenOptions && si->mode == EStartMode::NEW_GAME)
 	{
 		if(!si->mapGenOptions->checkOptions())
 			throw std::domain_error(VLC->generaltexth->translate("core.genrltxt.751"));
@@ -123,7 +137,12 @@ bool LobbyInfo::isClientHost(int clientId) const
 	return clientId == hostClientId;
 }
 
-std::set<PlayerColor> LobbyInfo::getAllClientPlayers(int clientId)
+bool LobbyInfo::isPlayerHost(const PlayerColor & color) const
+{
+	return vstd::contains(getAllClientPlayers(hostClientId), color);
+}
+
+std::set<PlayerColor> LobbyInfo::getAllClientPlayers(int clientId) const
 {
 	std::set<PlayerColor> players;
 	for(auto & elem : si->playerInfos)
