@@ -66,7 +66,9 @@ void CBufferedStream::ensureSize(si64 size)
 	{
 		si64 initialSize = buffer.size();
 		si64 currentStep = std::min<si64>(size, buffer.size());
-		vstd::amax(currentStep, 1024); // to avoid large number of calls at start
+		// to avoid large number of calls at start
+		// this is often used to load h3m map headers, most of which are ~300 bytes in size
+		vstd::amax(currentStep, 512);
 
 		buffer.resize(initialSize + currentStep);
 
@@ -162,9 +164,9 @@ si64 CCompressedStream::readMore(ui8 *data, si64 size)
 			break;
 		default:
 			if (inflateState->msg == nullptr)
-				throw std::runtime_error("Decompression error. Return code was " + std::to_string(ret));
+				throw DecompressionException("Error code " + std::to_string(ret));
 			else
-				throw std::runtime_error(std::string("Decompression error: ") + inflateState->msg);
+				throw DecompressionException(inflateState->msg);
 		}
 	}
 	while (!endLoop && inflateState->avail_out != 0 );
