@@ -16,6 +16,7 @@
 #include "../CGeneralTextHandler.h"
 #include "../gameState/CGameState.h"
 #include "../CPlayerState.h"
+#include "../MetaString.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
@@ -39,13 +40,14 @@ void CArmedInstance::randomizeArmy(FactionID type)
 // Take Angelic Alliance troop-mixing freedom of non-evil units into account.
 CSelector CArmedInstance::nonEvilAlignmentMixSelector = Selector::type()(BonusType::NONEVIL_ALIGNMENT_MIX);
 
-CArmedInstance::CArmedInstance()
-	:CArmedInstance(false)
+CArmedInstance::CArmedInstance(IGameCallback *cb)
+	:CArmedInstance(cb, false)
 {
 }
 
-CArmedInstance::CArmedInstance(bool isHypotetic):
-	CBonusSystemNode(isHypotetic),
+CArmedInstance::CArmedInstance(IGameCallback *cb, bool isHypothetic):
+	CGObjectInstance(cb),
+	CBonusSystemNode(isHypothetic),
 	nonEvilAlignmentMix(this, nonEvilAlignmentMixSelector),
 	battle(nullptr)
 {
@@ -99,23 +101,21 @@ void CArmedInstance::updateMoraleBonusFromArmy()
 			factionsInArmy -= mixableFactions - 1;
 	}
 
-	std::string description;
+	MetaString bonusDescription;
 
 	if(factionsInArmy == 1)
 	{
 		b->val = +1;
-		description = VLC->generaltexth->arraytxt[115]; //All troops of one alignment +1
-		description = description.substr(0, description.size()-3);//trim "+1"
+		bonusDescription.appendTextID("core.arraytxt.115"); //All troops of one alignment +1
 	}
 	else if (!factions.empty()) // no bonus from empty garrison
 	{
 		b->val = 2 - static_cast<si32>(factionsInArmy);
-		description = boost::str(boost::format(VLC->generaltexth->arraytxt[114]) % factionsInArmy % b->val); //Troops of %d alignments %d
-		description = description.substr(0, description.size()-3);//trim value
+		bonusDescription.appendTextID("core.arraytxt.114"); //Troops of %d alignments %d
+		bonusDescription.replaceNumber(factionsInArmy);
 	}
 	
-	boost::algorithm::trim(description);
-	b->description = description;
+	b->description = bonusDescription;
 
 	CBonusSystemNode::treeHasChanged();
 
@@ -125,8 +125,8 @@ void CArmedInstance::updateMoraleBonusFromArmy()
 	{
 		if(!undeadModifier)
 		{
-			undeadModifier = std::make_shared<Bonus>(BonusDuration::PERMANENT, BonusType::MORALE, BonusSource::ARMY, -1, BonusCustomSource::undeadMoraleDebuff, VLC->generaltexth->arraytxt[116]);
-			undeadModifier->description = undeadModifier->description.substr(0, undeadModifier->description.size()-2);//trim value
+			undeadModifier = std::make_shared<Bonus>(BonusDuration::PERMANENT, BonusType::MORALE, BonusSource::ARMY, -1, BonusCustomSource::undeadMoraleDebuff);
+			undeadModifier->description.appendTextID("core.arraytxt.116");
 			addNewBonus(undeadModifier);
 		}
 	}
