@@ -19,6 +19,8 @@ struct InfoWindow;
 class DLL_LINKAGE CGPandoraBox : public CRewardableObject
 {
 public:
+	using CRewardableObject::CRewardableObject;
+
 	MetaString message;
 
 	void initObj(CRandomGenerator & rand) override;
@@ -26,7 +28,7 @@ public:
 	void battleFinished(const CGHeroInstance *hero, const BattleResult &result) const override;
 	void blockingDialogAnswered(const CGHeroInstance *hero, ui32 answer) const override;
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CRewardableObject&>(*this);
 		h & message;
@@ -41,12 +43,14 @@ protected:
 class DLL_LINKAGE CGEvent : public CGPandoraBox  //event objects
 {
 public:
+	using CGPandoraBox::CGPandoraBox;
+
 	bool removeAfterVisit = false; //true if event is removed after occurring
 	std::set<PlayerColor> availableFor; //players whom this event is available for
 	bool computerActivate = false; //true if computer player can activate this event
 	bool humanActivate = false; //true if human player can activate this event
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CGPandoraBox &>(*this);
 		h & removeAfterVisit;
