@@ -24,6 +24,8 @@ using TTeleportExitsList = std::vector<std::pair<ObjectInstanceID, int3>>;
 class DLL_LINKAGE CTeamVisited: public CGObjectInstance
 {
 public:
+	using CGObjectInstance::CGObjectInstance;
+
 	std::set<PlayerColor> players; //players that visited this object
 
 	bool wasVisited (const CGHeroInstance * h) const override;
@@ -31,7 +33,7 @@ public:
 	bool wasVisited(const TeamID & team) const;
 	void setPropertyDer(ObjProperty what, ObjPropertyID identifier) override;
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CGObjectInstance&>(*this);
 		h & players;
@@ -41,12 +43,14 @@ public:
 class DLL_LINKAGE CGSignBottle : public CGObjectInstance //signs and ocean bottles
 {
 public:
+	using CGObjectInstance::CGObjectInstance;
+
 	MetaString message;
 
 	void onHeroVisit(const CGHeroInstance * h) const override;
 	void initObj(CRandomGenerator & rand) override;
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CGObjectInstance&>(*this);
 		h & message;
@@ -58,24 +62,30 @@ protected:
 class DLL_LINKAGE CGGarrison : public CArmedInstance
 {
 public:
+	using CArmedInstance::CArmedInstance;
+
 	bool removableUnits;
 
+	void initObj(CRandomGenerator &rand) override;
 	bool passableFor(PlayerColor color) const override;
 	void onHeroVisit(const CGHeroInstance * h) const override;
 	void battleFinished(const CGHeroInstance *hero, const BattleResult &result) const override;
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CArmedInstance&>(*this);
 		h & removableUnits;
 	}
 protected:
 	void serializeJsonOptions(JsonSerializeFormat & handler) override;
+	void addAntimagicGarrisonBonus();
 };
 
 class DLL_LINKAGE CGArtifact : public CArmedInstance
 {
 public:
+	using CArmedInstance::CArmedInstance;
+
 	CArtifactInstance * storedArtifact = nullptr;
 	MetaString message;
 
@@ -97,7 +107,7 @@ public:
 
 	ArtifactID getArtifact() const;
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CArmedInstance&>(*this);
 		h & message;
@@ -110,6 +120,8 @@ protected:
 class DLL_LINKAGE CGResource : public CArmedInstance
 {
 public:
+	using CArmedInstance::CArmedInstance;
+
 	static constexpr ui32 RANDOM_AMOUNT = 0;
 	ui32 amount = RANDOM_AMOUNT; //0 if random
 	
@@ -125,7 +137,7 @@ public:
 	void collectRes(const PlayerColor & player) const;
 	GameResID resourceID() const;
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CArmedInstance&>(*this);
 		h & amount;
@@ -146,6 +158,8 @@ public:
 	ResourceSet dailyIncome() const;
 
 private:
+	using CArmedInstance::CArmedInstance;
+
 	void onHeroVisit(const CGHeroInstance * h) const override;
 	void battleFinished(const CGHeroInstance *hero, const BattleResult &result) const override;
 	void blockingDialogAnswered(const CGHeroInstance *hero, ui32 answer) const override;
@@ -158,7 +172,7 @@ private:
 	std::string getHoverText(PlayerColor player) const override;
 
 public:
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CArmedInstance&>(*this);
 		h & producedResource;
@@ -179,7 +193,7 @@ struct DLL_LINKAGE TeleportChannel
 	std::vector<ObjectInstanceID> exits;
 	EPassability passability = EPassability::UNKNOWN;
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & entrances;
 		h & exits;
@@ -202,6 +216,8 @@ protected:
 	std::vector<ObjectInstanceID> getAllExits(bool excludeCurrent = false) const;
 
 public:
+	using CGObjectInstance::CGObjectInstance;
+
 	TeleportChannelID channel;
 
 	bool isEntrance() const;
@@ -216,7 +232,7 @@ public:
 	static std::vector<ObjectInstanceID> getPassableExits(CGameState * gs, const CGHeroInstance * h, std::vector<ObjectInstanceID> exits);
 	static bool isExitPassable(CGameState * gs, const CGHeroInstance * h, const CGObjectInstance * obj);
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & type;
 		h & channel;
@@ -234,7 +250,9 @@ protected:
 	void initObj(CRandomGenerator & rand) override;
 
 public:
-	template <typename Handler> void serialize(Handler &h, const int version)
+	using CGTeleport::CGTeleport;
+
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CGTeleport&>(*this);
 	}
@@ -246,9 +264,11 @@ class DLL_LINKAGE CGSubterraneanGate : public CGMonolith
 	void initObj(CRandomGenerator & rand) override;
 
 public:
-	static void postInit();
+	using CGMonolith::CGMonolith;
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	static void postInit(IGameCallback * cb);
+
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CGMonolith&>(*this);
 	}
@@ -261,7 +281,9 @@ class DLL_LINKAGE CGWhirlpool : public CGMonolith
 	static bool isProtected( const CGHeroInstance * h );
 
 public:
-	template <typename Handler> void serialize(Handler &h, const int version)
+	using CGMonolith::CGMonolith;
+
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CGMonolith&>(*this);
 	}
@@ -270,11 +292,13 @@ public:
 class DLL_LINKAGE CGSirens : public CGObjectInstance
 {
 public:
+	using CGObjectInstance::CGObjectInstance;
+
 	void onHeroVisit(const CGHeroInstance * h) const override;
 	std::string getHoverText(const CGHeroInstance * hero) const override;
 	void initObj(CRandomGenerator & rand) override;
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CGObjectInstance&>(*this);
 	}
@@ -283,6 +307,8 @@ public:
 class DLL_LINKAGE CGBoat : public CGObjectInstance, public CBonusSystemNode
 {
 public:
+	using CGObjectInstance::CGObjectInstance;
+
 	ui8 direction;
 	const CGHeroInstance *hero;  //hero on board
 	bool onboardAssaultAllowed; //if true, hero can attack units from transport
@@ -294,10 +320,10 @@ public:
 	AnimationPath overlayAnimation; //waves animations
 	std::array<AnimationPath, PlayerColor::PLAYER_LIMIT_I> flagAnimations;
 
-	CGBoat();
+	CGBoat(IGameCallback * cb);
 	bool isCoastVisitable() const override;
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CGObjectInstance&>(*this);
 		h & static_cast<CBonusSystemNode&>(*this);
@@ -325,7 +351,9 @@ protected:
 	BoatId getBoatType() const override;
 
 public:
-	template<typename Handler> void serialize(Handler & h, const int version)
+	using CGObjectInstance::CGObjectInstance;
+
+	template<typename Handler> void serialize(Handler & h)
 	{
 		h & static_cast<CGObjectInstance&>(*this);
 		h & createdBoat;
@@ -338,10 +366,12 @@ protected:
 class DLL_LINKAGE CGMagi : public CGObjectInstance
 {
 public:
+	using CGObjectInstance::CGObjectInstance;
+
 	void initObj(CRandomGenerator & rand) override;
 	void onHeroVisit(const CGHeroInstance * h) const override;
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CGObjectInstance&>(*this);
 	}
@@ -350,20 +380,20 @@ public:
 class DLL_LINKAGE CGDenOfthieves : public CGObjectInstance
 {
 	void onHeroVisit(const CGHeroInstance * h) const override;
+public:
+	using CGObjectInstance::CGObjectInstance;
 };
 
 class DLL_LINKAGE CGObelisk : public CTeamVisited
 {
 public:
-	static ui8 obeliskCount; //how many obelisks are on map
-	static std::map<TeamID, ui8> visited; //map: team_id => how many obelisks has been visited
+	using CTeamVisited::CTeamVisited;
 
 	void onHeroVisit(const CGHeroInstance * h) const override;
 	void initObj(CRandomGenerator & rand) override;
 	std::string getHoverText(PlayerColor player) const override;
-	static void reset();
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CTeamVisited&>(*this);
 	}
@@ -374,10 +404,12 @@ protected:
 class DLL_LINKAGE CGLighthouse : public CGObjectInstance
 {
 public:
+	using CGObjectInstance::CGObjectInstance;
+
 	void onHeroVisit(const CGHeroInstance * h) const override;
 	void initObj(CRandomGenerator & rand) override;
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CGObjectInstance&>(*this);
 	}
@@ -390,9 +422,9 @@ protected:
 class DLL_LINKAGE CGTerrainPatch : public CGObjectInstance
 {
 public:
-	CGTerrainPatch() = default;
+	using CGObjectInstance::CGObjectInstance;
 
-	virtual bool isTile2Terrain() const override
+	bool isTile2Terrain() const override
 	{
 		return true;
 	}
@@ -409,7 +441,9 @@ protected:
 	void fillUpgradeInfo(UpgradeInfo & info, const CStackInstance &stack) const override;
 
 public:
-	template <typename Handler> void serialize(Handler &h, const int version)
+	using CGObjectInstance::CGObjectInstance;
+
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & static_cast<CGObjectInstance&>(*this);
 		h & upgradeCostPercentage;
