@@ -10,8 +10,6 @@
 #include "StdInc.h"
 #include "CMapInfo.h"
 
-#include <vstd/DateUtils.h>
-
 #include "../filesystem/ResourcePath.h"
 #include "../StartInfo.h"
 #include "../GameConstants.h"
@@ -23,10 +21,13 @@
 #include "../filesystem/Filesystem.h"
 #include "../serializer/CLoadFile.h"
 #include "../CGeneralTextHandler.h"
+#include "../TextOperations.h"
 #include "../rmg/CMapGenOptions.h"
 #include "../CCreatureHandler.h"
 #include "../GameSettings.h"
 #include "../CHeroHandler.h"
+#include "../Languages.h"
+#include "../CConfigHandler.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
@@ -54,7 +55,7 @@ void CMapInfo::mapInit(const std::string & fname)
 
 void CMapInfo::saveInit(const ResourcePath & file)
 {
-	CLoadFile lf(*CResourceHandler::get()->getResourceName(file), MINIMAL_SERIALIZATION_VERSION);
+	CLoadFile lf(*CResourceHandler::get()->getResourceName(file), ESerializationVersion::MINIMAL);
 	lf.checkMagicBytes(SAVEGAME_MAGIC);
 
 	mapHeader = std::make_unique<CMapHeader>();
@@ -63,8 +64,8 @@ void CMapInfo::saveInit(const ResourcePath & file)
 	originalFileURI = file.getOriginalName();
 	fullFileURI = boost::filesystem::canonical(*CResourceHandler::get()->getResourceName(file)).string();
 	countPlayers();
-	std::time_t time = boost::filesystem::last_write_time(*CResourceHandler::get()->getResourceName(file));
-	date = vstd::getFormattedDateTime(time);
+	lastWrite = boost::filesystem::last_write_time(*CResourceHandler::get()->getResourceName(file));
+	date = TextOperations::getFormattedDateTimeLocal(lastWrite);
 
 	// We absolutely not need this data for lobby and server will read it from save
 	// FIXME: actually we don't want them in CMapHeader!
