@@ -35,6 +35,34 @@ class SpellID;
 class PlayerColor;
 class int3;
 
+enum class EQuestMission;
+
+enum class EVictoryConditionType : int8_t
+{
+	WINSTANDARD = -1,
+	ARTIFACT = 0,
+	GATHERTROOP = 1,
+	GATHERRESOURCE = 2,
+	BUILDCITY = 3,
+	BUILDGRAIL = 4,
+	BEATHERO = 5,
+	CAPTURECITY = 6,
+	BEATMONSTER = 7,
+	TAKEDWELLINGS = 8,
+	TAKEMINES = 9,
+	TRANSPORTITEM = 10,
+	HOTA_ELIMINATE_ALL_MONSTERS = 11,
+	HOTA_SURVIVE_FOR_DAYS = 12
+};
+
+enum class ELossConditionType : int8_t
+{
+	LOSSSTANDARD = -1,
+	LOSSCASTLE = 0,
+	LOSSHERO = 1,
+	TIMEEXPIRES = 2
+};
+
 class DLL_LINKAGE CMapLoaderH3M : public IMapLoader
 {
 public:
@@ -55,7 +83,7 @@ public:
 	 *
 	 * @return a unique ptr of the loaded map class
 	 */
-	std::unique_ptr<CMap> loadMap() override;
+	std::unique_ptr<CMap> loadMap(IGameCallback * cb) override;
 
 	/**
 	 * Loads the VCMI/H3 map header.
@@ -204,7 +232,7 @@ private:
 	 *
 	 * @param guard the quest guard where that quest should be applied to
 	 */
-	int readQuest(IQuestObject * guard, const int3 & position);
+	EQuestMission readQuest(IQuestObject * guard, const int3 & position);
 
 	void readSeerHutQuest(CGSeerHut * hut, const int3 & position, const ObjectInstanceID & idToBeGiven);
 
