@@ -62,7 +62,7 @@ struct DLL_LINKAGE PackageApplied : public CPackForClient
 	ui32 requestID = 0; //an ID given by client to the request that was applied
 	PlayerColor player;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & result;
 		h & packType;
@@ -73,7 +73,7 @@ struct DLL_LINKAGE PackageApplied : public CPackForClient
 
 struct DLL_LINKAGE SystemMessage : public CPackForClient
 {
-	explicit SystemMessage(std::string Text)
+	explicit SystemMessage(MetaString Text)
 		: text(std::move(Text))
 	{
 	}
@@ -81,9 +81,9 @@ struct DLL_LINKAGE SystemMessage : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	std::string text;
+	MetaString text;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & text;
 	}
@@ -100,7 +100,7 @@ struct DLL_LINKAGE PlayerBlocked : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & reason;
 		h & startOrEnd;
@@ -118,7 +118,7 @@ struct DLL_LINKAGE PlayerCheated : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & player;
 		h & losingCheatCode;
@@ -133,7 +133,7 @@ struct DLL_LINKAGE TurnTimeUpdate : public CPackForClient
 	PlayerColor player;
 	TurnTimerInfo turnTimer;
 		
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & player;
 		h & turnTimer;
@@ -148,7 +148,7 @@ struct DLL_LINKAGE PlayerStartsTurn : public Query
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & queryID;
 		h & player;
@@ -164,7 +164,7 @@ struct DLL_LINKAGE DaysWithoutTown : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & player;
 		h & daysWithoutCastle;
@@ -179,7 +179,7 @@ struct DLL_LINKAGE EntitiesChanged : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & changes;
 	}
@@ -195,7 +195,7 @@ struct DLL_LINKAGE SetResources : public CPackForClient
 	PlayerColor player;
 	ResourceSet res; //res[resid] => res amount
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & abs;
 		h & player;
@@ -214,7 +214,7 @@ struct DLL_LINKAGE SetPrimSkill : public CPackForClient
 	PrimarySkill which = PrimarySkill::ATTACK;
 	si64 val = 0;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & abs;
 		h & id;
@@ -234,7 +234,7 @@ struct DLL_LINKAGE SetSecSkill : public CPackForClient
 	SecondarySkill which;
 	ui16 val = 0;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & abs;
 		h & id;
@@ -250,14 +250,15 @@ struct DLL_LINKAGE HeroVisitCastle : public CPackForClient
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	ui8 flags = 0; //1 - start
-	ObjectInstanceID tid, hid;
+	ObjectInstanceID tid;
+	ObjectInstanceID hid;
 
 	bool start() const //if hero is entering castle (if false - leaving)
 	{
 		return flags & 1;
 	}
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & flags;
 		h & tid;
@@ -275,7 +276,7 @@ struct DLL_LINKAGE ChangeSpells : public CPackForClient
 	ObjectInstanceID hid;
 	std::set<SpellID> spells;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & learn;
 		h & hid;
@@ -293,7 +294,7 @@ struct DLL_LINKAGE SetMana : public CPackForClient
 	si32 val = 0;
 	bool absolute = true;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & val;
 		h & hid;
@@ -311,7 +312,7 @@ struct DLL_LINKAGE SetMovePoints : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & val;
 		h & hid;
@@ -330,7 +331,7 @@ struct DLL_LINKAGE FoWChange : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & tiles;
 		h & player;
@@ -352,16 +353,18 @@ struct DLL_LINKAGE SetAvailableHero : public CPackForClient
 	PlayerColor player;
 	HeroTypeID hid; //HeroTypeID::NONE if no hero
 	CSimpleArmy army;
+	bool replenishPoints;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & slotID;
 		h & roleID;
 		h & player;
 		h & hid;
 		h & army;
+		h & replenishPoints;
 	}
 };
 
@@ -379,15 +382,13 @@ struct DLL_LINKAGE GiveBonus : public CPackForClient
 	ETarget who = ETarget::OBJECT;
 	VariantIdentifier<ObjectInstanceID, PlayerColor, BattleID> id;
 	Bonus bonus;
-	MetaString bdescr;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & bonus;
 		h & id;
-		h & bdescr;
 		h & who;
 		assert(id.getNum() != -1);
 	}
@@ -406,7 +407,7 @@ struct DLL_LINKAGE ChangeObjPos : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & objid;
 		h & nPos;
@@ -422,7 +423,7 @@ struct DLL_LINKAGE PlayerEndsTurn : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & player;
 	}
@@ -437,7 +438,7 @@ struct DLL_LINKAGE PlayerEndsGame : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & player;
 		h & victoryLossCheckResult;
@@ -453,7 +454,7 @@ struct DLL_LINKAGE PlayerReinitInterface : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & players;
 		h & playerConnectionId;
@@ -481,7 +482,7 @@ struct DLL_LINKAGE RemoveBonus : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & source;
 		h & id;
@@ -505,7 +506,7 @@ struct DLL_LINKAGE SetCommanderProperty : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & heroid;
 		h & which;
@@ -524,7 +525,7 @@ struct DLL_LINKAGE AddQuest : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & player;
 		h & quest;
@@ -538,7 +539,7 @@ struct DLL_LINKAGE UpdateArtHandlerLists : public CPackForClient
 	void applyGs(CGameState * gs) const;
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & allocatedArtifacts;
 	}
@@ -551,7 +552,7 @@ struct DLL_LINKAGE UpdateMapEvents : public CPackForClient
 	void applyGs(CGameState * gs) const;
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & events;
 	}
@@ -565,7 +566,7 @@ struct DLL_LINKAGE UpdateCastleEvents : public CPackForClient
 	void applyGs(CGameState * gs) const;
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & town;
 		h & events;
@@ -580,7 +581,7 @@ struct DLL_LINKAGE ChangeFormation : public CPackForClient
 	void applyGs(CGameState * gs) const;
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & hid;
 		h & formation;
@@ -605,7 +606,7 @@ struct DLL_LINKAGE RemoveObject : public CPackForClient
 	/// Player that initiated this action, if any
 	PlayerColor initiator;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & objectID;
 		h & initiator;
@@ -629,7 +630,8 @@ struct DLL_LINKAGE TryMoveHero : public CPackForClient
 	ObjectInstanceID id;
 	ui32 movePoints = 0;
 	EResult result = FAILED; //uses EResult
-	int3 start, end; //h3m format
+	int3 start; //h3m format
+	int3 end;
 	std::unordered_set<int3> fowRevealed; //revealed tiles
 	std::optional<int3> attackedFrom; // Set when stepping into endangered tile.
 
@@ -640,7 +642,7 @@ struct DLL_LINKAGE TryMoveHero : public CPackForClient
 		return result != SUCCESS && result != EMBARK && result != DISEMBARK && result != TELEPORTATION;
 	}
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & id;
 		h & result;
@@ -662,7 +664,7 @@ struct DLL_LINKAGE NewStructures : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & tid;
 		h & bid;
@@ -680,7 +682,7 @@ struct DLL_LINKAGE RazeStructures : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & tid;
 		h & bid;
@@ -697,7 +699,7 @@ struct DLL_LINKAGE SetAvailableCreatures : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & tid;
 		h & creatures;
@@ -708,11 +710,13 @@ struct DLL_LINKAGE SetHeroesInTown : public CPackForClient
 {
 	void applyGs(CGameState * gs) const;
 
-	ObjectInstanceID tid, visiting, garrison; //id of town, visiting hero, hero in garrison
+	ObjectInstanceID tid; //id of town
+	ObjectInstanceID visiting; //id of visiting hero
+	ObjectInstanceID garrison; //id of hero in garrison
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & tid;
 		h & visiting;
@@ -732,7 +736,7 @@ struct DLL_LINKAGE HeroRecruited : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & hid;
 		h & tid;
@@ -752,7 +756,7 @@ struct DLL_LINKAGE GiveHero : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & id;
 		h & boatId;
@@ -768,7 +772,7 @@ struct DLL_LINKAGE OpenWindow : public Query
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & queryID;
 		h & window;
@@ -794,10 +798,10 @@ struct DLL_LINKAGE NewObject : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & ID;
-		subID.serializeIdentifier(h, ID, version);
+		subID.serializeIdentifier(h, ID);
 		h & targetPos;
 		h & initiator;
 	}
@@ -813,7 +817,7 @@ struct DLL_LINKAGE SetAvailableArtifacts : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & id;
 		h & arts;
@@ -835,7 +839,7 @@ struct DLL_LINKAGE ChangeStackCount : CGarrisonOperationPack
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & army;
 		h & slot;
@@ -854,7 +858,7 @@ struct DLL_LINKAGE SetStackType : CGarrisonOperationPack
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & army;
 		h & slot;
@@ -870,7 +874,7 @@ struct DLL_LINKAGE EraseStack : CGarrisonOperationPack
 	void applyGs(CGameState * gs);
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & army;
 		h & slot;
@@ -887,7 +891,7 @@ struct DLL_LINKAGE SwapStacks : CGarrisonOperationPack
 	void applyGs(CGameState * gs);
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & srcArmy;
 		h & dstArmy;
@@ -906,7 +910,7 @@ struct DLL_LINKAGE InsertNewStack : CGarrisonOperationPack
 	void applyGs(CGameState * gs);
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & army;
 		h & slot;
@@ -928,7 +932,7 @@ struct DLL_LINKAGE RebalanceStacks : CGarrisonOperationPack
 	void applyGs(CGameState * gs);
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & srcArmy;
 		h & dstArmy;
@@ -946,7 +950,7 @@ struct DLL_LINKAGE BulkRebalanceStacks : CGarrisonOperationPack
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler>
-	void serialize(Handler & h, const int version)
+	void serialize(Handler & h)
 	{
 		h & moves;
 	}
@@ -961,7 +965,7 @@ struct DLL_LINKAGE BulkSmartRebalanceStacks : CGarrisonOperationPack
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler>
-	void serialize(Handler & h, const int version)
+	void serialize(Handler & h)
 	{
 		h & moves;
 		h & changes;
@@ -987,7 +991,7 @@ struct DLL_LINKAGE PutArtifact : CArtifactOperationPack
 	void applyGs(CGameState * gs);
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & al;
 		h & askAssemble;
@@ -1002,7 +1006,7 @@ struct DLL_LINKAGE NewArtifact : public CArtifactOperationPack
 	void applyGs(CGameState * gs);
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & art;
 	}
@@ -1015,7 +1019,7 @@ struct DLL_LINKAGE EraseArtifact : CArtifactOperationPack
 	void applyGs(CGameState * gs);
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & al;
 	}
@@ -1024,18 +1028,21 @@ struct DLL_LINKAGE EraseArtifact : CArtifactOperationPack
 struct DLL_LINKAGE MoveArtifact : CArtifactOperationPack
 {
 	MoveArtifact() = default;
-	MoveArtifact(ArtifactLocation * src, ArtifactLocation * dst, bool askAssemble = true)
-		: src(*src), dst(*dst), askAssemble(askAssemble)
+	MoveArtifact(const PlayerColor & interfaceOwner, const ArtifactLocation & src, const ArtifactLocation & dst, bool askAssemble = true)
+		: interfaceOwner(interfaceOwner), src(src), dst(dst), askAssemble(askAssemble)
 	{
 	}
-	ArtifactLocation src, dst;
+	PlayerColor interfaceOwner;
+	ArtifactLocation src;
+	ArtifactLocation dst;
 	bool askAssemble = true;
 
 	void applyGs(CGameState * gs);
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
+		h & interfaceOwner;
 		h & src;
 		h & dst;
 		h & askAssemble;
@@ -1055,20 +1062,22 @@ struct DLL_LINKAGE BulkMoveArtifacts : CArtifactOperationPack
 			, dstPos(dstPos)
 		{
 		}
-		template <typename Handler> void serialize(Handler & h, const int version)
+		template <typename Handler> void serialize(Handler & h)
 		{
 			h & srcPos;
 			h & dstPos;
 		}
 	};
 
+	PlayerColor interfaceOwner;
 	ObjectInstanceID srcArtHolder;
 	ObjectInstanceID dstArtHolder;
 	std::optional<SlotID> srcCreature;
 	std::optional<SlotID> dstCreature;
 
 	BulkMoveArtifacts()
-		: srcArtHolder(ObjectInstanceID::NONE)
+		: interfaceOwner(PlayerColor::NEUTRAL)
+		, srcArtHolder(ObjectInstanceID::NONE)
 		, dstArtHolder(ObjectInstanceID::NONE)
 		, swap(false)
 		, askAssemble(false)
@@ -1076,9 +1085,10 @@ struct DLL_LINKAGE BulkMoveArtifacts : CArtifactOperationPack
 		, dstCreature(std::nullopt)
 	{
 	}
-	BulkMoveArtifacts(const ObjectInstanceID srcArtHolder, const ObjectInstanceID dstArtHolder, bool swap)
-		: srcArtHolder(std::move(srcArtHolder))
-		, dstArtHolder(std::move(dstArtHolder))
+	BulkMoveArtifacts(const PlayerColor & interfaceOwner, const ObjectInstanceID srcArtHolder, const ObjectInstanceID dstArtHolder, bool swap)
+		: interfaceOwner(interfaceOwner)
+		, srcArtHolder(srcArtHolder)
+		, dstArtHolder(dstArtHolder)
 		, swap(swap)
 		, askAssemble(false)
 		, srcCreature(std::nullopt)
@@ -1095,8 +1105,9 @@ struct DLL_LINKAGE BulkMoveArtifacts : CArtifactOperationPack
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
+		h & interfaceOwner;
 		h & artsPack0;
 		h & artsPack1;
 		h & srcArtHolder;
@@ -1111,13 +1122,13 @@ struct DLL_LINKAGE BulkMoveArtifacts : CArtifactOperationPack
 struct DLL_LINKAGE AssembledArtifact : CArtifactOperationPack
 {
 	ArtifactLocation al; //where assembly will be put
-	CArtifact * builtArt;
+	const CArtifact * builtArt;
 
 	void applyGs(CGameState * gs);
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & al;
 		h & builtArt;
@@ -1132,7 +1143,7 @@ struct DLL_LINKAGE DisassembledArtifact : CArtifactOperationPack
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & al;
 	}
@@ -1150,7 +1161,7 @@ struct DLL_LINKAGE HeroVisit : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & player;
 		h & heroId;
@@ -1169,9 +1180,10 @@ struct DLL_LINKAGE NewTurn : public CPackForClient
 
 	struct Hero
 	{
-		ObjectInstanceID id;
-		ui32 move, mana; //id is a general serial id
-		template <typename Handler> void serialize(Handler & h, const int version)
+		ObjectInstanceID id; //id is a general serial id
+		ui32 move;
+		ui32 mana;
+		template <typename Handler> void serialize(Handler & h)
 		{
 			h & id;
 			h & move;
@@ -1189,7 +1201,7 @@ struct DLL_LINKAGE NewTurn : public CPackForClient
 
 	NewTurn() = default;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & heroes;
 		h & cres;
@@ -1210,7 +1222,7 @@ struct DLL_LINKAGE InfoWindow : public CPackForClient //103  - displays simple i
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & type;
 		h & text;
@@ -1233,7 +1245,7 @@ struct DLL_LINKAGE SetObjectProperty : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & id;
 		h & what;
@@ -1268,7 +1280,7 @@ struct DLL_LINKAGE ChangeObjectVisitors : public CPackForClient
 	{
 	}
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & object;
 		h & hero;
@@ -1276,6 +1288,30 @@ struct DLL_LINKAGE ChangeObjectVisitors : public CPackForClient
 	}
 };
 
+struct DLL_LINKAGE ChangeArtifactsCostume : public CPackForClient
+{
+	std::map<ArtifactPosition, ArtifactID> costumeSet;
+	uint32_t costumeIdx = 0;
+	const PlayerColor player = PlayerColor::NEUTRAL;
+
+	void applyGs(CGameState * gs) const;
+	void visitTyped(ICPackVisitor & visitor) override;
+
+	ChangeArtifactsCostume() = default;
+	ChangeArtifactsCostume(const PlayerColor & player, const uint32_t costumeIdx)
+		: costumeIdx(costumeIdx)
+		, player(player)
+	{
+	}
+
+	template <typename Handler> void serialize(Handler & h)
+	{
+		h & costumeSet;
+		h & costumeIdx;
+		h & player;
+	}
+};
+
 struct DLL_LINKAGE HeroLevelUp : public Query
 {
 	PlayerColor player;
@@ -1288,7 +1324,7 @@ struct DLL_LINKAGE HeroLevelUp : public Query
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & queryID;
 		h & player;
@@ -1309,7 +1345,7 @@ struct DLL_LINKAGE CommanderLevelUp : public Query
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & queryID;
 		h & player;
@@ -1323,7 +1359,7 @@ struct DLL_LINKAGE CommanderLevelUp : public Query
 //Until sending reply player won't be allowed to take any actions
 struct DLL_LINKAGE BlockingDialog : public Query
 {
-	enum { ALLOW_CANCEL = 1, SELECTION = 2 };
+	enum { ALLOW_CANCEL = 1, SELECTION = 2, SAFE_TO_AUTOACCEPT = 4 };
 	MetaString text;
 	std::vector<Component> components;
 	PlayerColor player;
@@ -1339,6 +1375,11 @@ struct DLL_LINKAGE BlockingDialog : public Query
 		return flags & SELECTION;
 	}
 
+	bool safeToAutoaccept() const
+	{
+		return flags & SAFE_TO_AUTOACCEPT;
+	}
+
 	BlockingDialog(bool yesno, bool Selection)
 	{
 		if(yesno) flags |= ALLOW_CANCEL;
@@ -1348,7 +1389,7 @@ struct DLL_LINKAGE BlockingDialog : public Query
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & queryID;
 		h & text;
@@ -1361,12 +1402,13 @@ struct DLL_LINKAGE BlockingDialog : public Query
 
 struct DLL_LINKAGE GarrisonDialog : public Query
 {
-	ObjectInstanceID objid, hid;
+	ObjectInstanceID objid;
+	ObjectInstanceID hid;
 	bool removableUnits = false;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & queryID;
 		h & objid;
@@ -1384,7 +1426,7 @@ struct DLL_LINKAGE ExchangeDialog : public Query
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & queryID;
 		h & player;
@@ -1409,7 +1451,7 @@ struct DLL_LINKAGE TeleportDialog : public Query
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & queryID;
 		h & hero;
@@ -1429,7 +1471,7 @@ struct DLL_LINKAGE MapObjectSelectDialog : public Query
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & queryID;
 		h & player;
@@ -1444,7 +1486,7 @@ struct DLL_LINKAGE AdvmapSpellCast : public CPackForClient
 {
 	ObjectInstanceID casterID;
 	SpellID spellID;
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & casterID;
 		h & spellID;
@@ -1461,7 +1503,7 @@ struct DLL_LINKAGE ShowWorldViewEx : public CPackForClient
 
 	std::vector<ObjectPosInfo> objectPositions;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & player;
 		h & showTerrain;
@@ -1485,7 +1527,7 @@ struct DLL_LINKAGE PlayerMessageClient : public CPackForClient
 	PlayerColor player;
 	std::string text;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & player;
 		h & text;
@@ -1500,7 +1542,7 @@ struct DLL_LINKAGE CenterView : public CPackForClient
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	template <typename Handler> void serialize(Handler & h, const int version)
+	template <typename Handler> void serialize(Handler & h)
 	{
 		h & pos;
 		h & player;
