@@ -17,6 +17,7 @@
 #include "CRandomGenerator.h"
 #include "../VCMI_Lib.h"
 #include "../CTownHandler.h"
+#include "serializer/JsonSerializeFormat.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
@@ -388,6 +389,13 @@ void CMapGenOptions::setStartingTownForPlayer(const PlayerColor & color, Faction
 	it->second.setStartingTown(town);
 }
 
+void CMapGenOptions::setStartingHeroForPlayer(const PlayerColor & color, HeroTypeID hero)
+{
+	auto it = players.find(color);
+	assert(it != players.end());
+	it->second.setStartingHero(hero);
+}
+
 void CMapGenOptions::setPlayerTypeForStandardPlayer(const PlayerColor & color, EPlayerType playerType)
 {
 	// FIXME: Why actually not set it to COMP_ONLY? Ie. when swapping human to another color?
@@ -746,7 +754,7 @@ const CRmgTemplate * CMapGenOptions::getPossibleTemplate(CRandomGenerator & rand
 	return *RandomGeneratorUtil::nextItem(templates, rand);
 }
 
-CMapGenOptions::CPlayerSettings::CPlayerSettings() : color(0), startingTown(FactionID::RANDOM), playerType(EPlayerType::AI), team(TeamID::NO_TEAM)
+CMapGenOptions::CPlayerSettings::CPlayerSettings() : color(0), startingTown(FactionID::RANDOM), startingHero(HeroTypeID::RANDOM), playerType(EPlayerType::AI), team(TeamID::NO_TEAM)
 {
 
 }
@@ -778,6 +786,17 @@ void CMapGenOptions::CPlayerSettings::setStartingTown(FactionID value)
 	startingTown = value;
 }
 
+HeroTypeID CMapGenOptions::CPlayerSettings::getStartingHero() const
+{
+	return startingHero;
+}
+
+void CMapGenOptions::CPlayerSettings::setStartingHero(HeroTypeID value)
+{
+	assert(value == HeroTypeID::RANDOM || value.toEntity(VLC) != nullptr);
+	startingHero = value;
+}
+
 EPlayerType CMapGenOptions::CPlayerSettings::getPlayerType() const
 {
 	return playerType;
@@ -798,4 +817,35 @@ void CMapGenOptions::CPlayerSettings::setTeam(const TeamID & value)
 	team = value;
 }
 
+void CMapGenOptions::serializeJson(JsonSerializeFormat & handler)
+{
+	handler.serializeInt("width", width);
+	handler.serializeInt("height", height);
+	handler.serializeBool("haswoLevels", hasTwoLevels);
+	handler.serializeInt("humanOrCpuPlayerCount", humanOrCpuPlayerCount);
+	handler.serializeInt("teamCount", teamCount);
+	handler.serializeInt("compOnlyPlayerCount", compOnlyPlayerCount);
+	handler.serializeInt("compOnlyTeamCount", compOnlyTeamCount);
+	handler.serializeInt("waterContent", waterContent);
+	handler.serializeInt("monsterStrength", monsterStrength);
+
+	std::string templateName;
+	if(mapTemplate && handler.saving)
+	{
+		templateName = mapTemplate->getId();
+	}
+	handler.serializeString("templateName", templateName);
+	if(!handler.saving)
+	{
+		setMapTemplate(templateName);
+	}
+
+	handler.serializeIdArray("roads", enabledRoads);
+	if (!handler.saving)
+	{
+		// Player settings won't be saved
+		resetPlayersMap();
+	}
+}
+
 VCMI_LIB_NAMESPACE_END
