@@ -45,6 +45,11 @@ public:
 		FactionID getStartingTown() const;
 		void setStartingTown(FactionID value);
 
+		/// The starting hero of the player ranging from 0 to hero max count or RANDOM_HERO.
+		/// The default value is RANDOM_HERO
+		HeroTypeID getStartingHero() const;
+		void setStartingHero(HeroTypeID value);
+
 		/// The default value is EPlayerType::AI.
 		EPlayerType getPlayerType() const;
 		void setPlayerType(EPlayerType value);
@@ -56,18 +61,22 @@ public:
 	private:
 		PlayerColor color;
 		FactionID startingTown;
+		HeroTypeID startingHero;
 		EPlayerType playerType;
 		TeamID team;
 
 	public:
 		template <typename Handler>
-		void serialize(Handler & h, const int version)
+		void serialize(Handler & h)
 		{
 			h & color;
 			h & startingTown;
 			h & playerType;
-			if(version >= 806)
-				h & team;
+			h & team;
+			if (h.version >= Handler::Version::RELEASE_143)
+				h & startingHero;
+			else
+				startingHero = HeroTypeID::RANDOM;
 		}
 	};
 
@@ -120,6 +129,7 @@ public:
 	const std::map<PlayerColor, CPlayerSettings> & getPlayersSettings() const;
 	const std::map<PlayerColor, CPlayerSettings> & getSavedPlayersMap() const;
 	void setStartingTownForPlayer(const PlayerColor & color, FactionID town);
+	void setStartingHeroForPlayer(const PlayerColor & color, HeroTypeID hero);
 	/// Sets a player type for a standard player. A standard player is the opposite of a computer only player. The
 	/// values which can be chosen for the player type are EPlayerType::AI or EPlayerType::HUMAN.
 	void setPlayerTypeForStandardPlayer(const PlayerColor & color, EPlayerType playerType);
@@ -157,9 +167,13 @@ private:
 	void updatePlayers();
 	const CRmgTemplate * getPossibleTemplate(CRandomGenerator & rand) const;
 
-	si32 width, height;
+	si32 width;
+	si32 height;
 	bool hasTwoLevels;
-	si8 humanOrCpuPlayerCount, teamCount, compOnlyPlayerCount, compOnlyTeamCount;
+	si8 humanOrCpuPlayerCount;
+	si8 teamCount;
+	si8 compOnlyPlayerCount;
+	si8 compOnlyTeamCount;
 	EWaterContent::EWaterContent waterContent;
 	EMonsterStrength::EMonsterStrength monsterStrength;
 	std::map<PlayerColor, CPlayerSettings> players;
@@ -171,7 +185,7 @@ private:
 
 public:
 	template <typename Handler>
-	void serialize(Handler & h, const int version)
+	void serialize(Handler & h)
 	{
 		h & width;
 		h & height;
@@ -188,17 +202,16 @@ public:
 		{
 			templateName = mapTemplate->getId();
 		}
-		if(version >= 806)
+		h & templateName;
+		if(!h.saving)
 		{
-			h & templateName;
-			if(!h.saving)
-			{
-				setMapTemplate(templateName);
-			}
-			
-			h & enabledRoads;
+			setMapTemplate(templateName);
 		}
+
+		h & enabledRoads;
 	}
+
+	void serializeJson(JsonSerializeFormat & handler);
 };
 
 VCMI_LIB_NAMESPACE_END
