@@ -33,9 +33,11 @@ public:
 	
 protected:
 	void collectNeighbourZones();
+	std::pair<Zone::Lock, Zone::Lock> lockZones(std::shared_ptr<Zone> otherZone);
 
 protected:
-	std::vector<rmg::ZoneConnection> dConnections, dCompleted;
+	std::vector<rmg::ZoneConnection> dConnections;
+	std::vector<rmg::ZoneConnection> dCompleted;
 	std::map<TRmgTemplateZoneId, rmg::Tileset> dNeighbourZones;
 };
 
