@@ -20,10 +20,12 @@
 #include "../../battle/CBattleInfoCallback.h"
 #include "../../networkPacks/PacksForClientBattle.h"
 #include "../../CGeneralTextHandler.h"
+#include "../../Languages.h"
 #include "../../serializer/JsonSerializeFormat.h"
 
 #include <vcmi/spells/Spell.h>
 
+
 VCMI_LIB_NAMESPACE_BEGIN
 
 namespace spells
@@ -152,6 +154,16 @@ void Damage::describeEffect(std::vector<MetaString> & log, const Mechanics * m,
 		m->caster->getCasterName(line);
 		log.push_back(line);
 	}
+	else if(m->getSpell()->getJsonKey().find("accurateShot") != std::string::npos && !multiple)
+	{
+		MetaString line;
+		std::string preferredLanguage = VLC->generaltexth->getPreferredLanguage();
+		std::string textID = "vcmi.battleWindow.accurateShot.resultDescription";
+		line.appendTextID(Languages::getPluralFormTextID( preferredLanguage, kills, textID));
+		line.replaceNumber(kills);
+		firstTarget->addNameReplacement(line, kills != 1);
+		log.push_back(line);
+	}
 	else if(m->getSpellIndex() == SpellID::THUNDERBOLT && !multiple)
 	{
 		{
