@@ -19,7 +19,7 @@
 #include "../lib/vcmi_endian.h"
 #include "../lib/filesystem/Filesystem.h"
 #include "../lib/filesystem/ISimpleResourceLoader.h"
-#include "../lib/JsonNode.h"
+#include "../lib/json/JsonUtils.h"
 #include "../lib/CRandomGenerator.h"
 #include "../lib/VCMIDirs.h"
 
@@ -62,7 +62,9 @@ class ImageLoader
 	QImage * image;
 	ui8 * lineStart;
 	ui8 * position;
-	QPoint spriteSize, margins, fullSize;
+	QPoint spriteSize;
+	QPoint margins;
+	QPoint fullSize;
 public:
 	//load size raw pixels from data
 	inline void Load(size_t size, const ui8 * data);
@@ -159,7 +161,7 @@ DefFile::DefFile(std::string Name):
 	#endif // 0
 
 	//First 8 colors in def palette used for transparency
-	static QRgb H3Palette[8] =
+	constexpr std::array H3Palette =
 	{
 		qRgba(0, 0, 0,   0), // 100% - transparency
 		qRgba(0, 0, 0,  32), //  75% - shadow border,
@@ -301,7 +303,7 @@ std::shared_ptr<QImage> DefFile::loadFrame(size_t frame, size_t group) const
 	const ui32 BaseOffset = currentOffset;
 
 	
-	std::shared_ptr<QImage> img = std::make_shared<QImage>(sprite.fullWidth, sprite.fullHeight, QImage::Format_Indexed8);
+	auto img = std::make_shared<QImage>(sprite.fullWidth, sprite.fullHeight, QImage::Format_Indexed8);
 	if(!img)
 		throw std::runtime_error("Image memory cannot be allocated");
 	
@@ -597,7 +599,7 @@ void Animation::init()
 		std::unique_ptr<ui8[]> textData(new ui8[stream->getSize()]);
 		stream->read(textData.get(), stream->getSize());
 
-		const JsonNode config((char*)textData.get(), stream->getSize());
+		const JsonNode config(reinterpret_cast<const std::byte*>(textData.get()), stream->getSize());
 
 		initFromJson(config);
 	}
@@ -608,7 +610,7 @@ void Animation::initFromJson(const JsonNode & config)
 	std::string basepath;
 	basepath = config["basepath"].String();
 
-	JsonNode base(JsonNode::JsonType::DATA_STRUCT);
+	JsonNode base;
 	base["margins"] = config["margins"];
 	base["width"] = config["width"];
 	base["height"] = config["height"];
@@ -620,7 +622,7 @@ void Animation::initFromJson(const JsonNode & config)
 
 		for(const JsonNode & frame : group["frames"].Vector())
 		{
-			JsonNode toAdd(JsonNode::JsonType::DATA_STRUCT);
+			JsonNode toAdd;
 			JsonUtils::inherit(toAdd, base);
 			toAdd["file"].String() = basepath + frame.String();
 			source[groupID].push_back(toAdd);
@@ -635,7 +637,7 @@ void Animation::initFromJson(const JsonNode & config)
 		if (source[group].size() <= frame)
 			source[group].resize(frame+1);
 
-		JsonNode toAdd(JsonNode::JsonType::DATA_STRUCT);
+		JsonNode toAdd;
 		JsonUtils::inherit(toAdd, base);
 		toAdd["file"].String() = basepath + node["file"].String();
 		source[group][frame] = toAdd;
