@@ -128,8 +128,7 @@ protected:
 		{
 			itemKey = keyItems[key];
 			table->setItem(table->row(itemKey), 1, itemValue);
-			if(delegate)
-				table->setItemDelegateForRow(table->row(itemKey), delegate);
+			table->setItemDelegateForRow(table->row(itemKey), delegate);
 		}
 		else
 		{
@@ -139,8 +138,7 @@ protected:
 			table->setRowCount(row + 1);
 			table->setItem(row, 0, itemKey);
 			table->setItem(row, 1, itemValue);
-			if(delegate)
-				table->setItemDelegateForRow(row, delegate);
+			table->setItemDelegateForRow(row, delegate);
 			++row;
 		}
 		itemKey->setFlags(restricted ? Qt::NoItemFlags : Qt::ItemIsEnabled);
@@ -173,3 +171,11 @@ public:
 	
 	QList<std::pair<QString, QVariant>> options;
 };
+
+
+class OwnerDelegate : public InspectorDelegate
+{
+	Q_OBJECT
+public:
+	OwnerDelegate(MapController & controller, bool addNeutral = true);
+};
\ No newline at end of file
