@@ -90,6 +90,9 @@ void MainWindow::loadUserSettings()
 	{
 		move(position);
 	}
+	lastSavingDir = s.value(lastDirectorySetting).toString();
+	if(lastSavingDir.isEmpty())
+		lastSavingDir = QString::fromStdString(VCMIDirs::get().userDataPath().make_preferred().string());
 }
 
 void MainWindow::saveUserSettings()
@@ -97,6 +100,7 @@ void MainWindow::saveUserSettings()
 	QSettings s(Ui::teamName, Ui::appName);
 	s.setValue(mainWindowSizeSetting, size());
 	s.setValue(mainWindowPositionSetting, pos());
+	s.setValue(lastDirectorySetting, lastSavingDir);
 }
 
 void MainWindow::parseCommandLine(ExtractionOptions & extractionOptions)
@@ -177,11 +181,11 @@ MainWindow::MainWindow(QWidget* parent) :
 	logGlobal->info("The log file will be saved to %s", logPath);
 
 	//init
-	preinitDLL(::console, false, extractionOptions.extractArchives);
+	preinitDLL(::console, extractionOptions.extractArchives);
 
 	// Initialize logging based on settings
 	logConfig->configure();
-	logGlobal->debug("settings = %s", settings.toJsonNode().toJson());
+	logGlobal->debug("settings = %s", settings.toJsonNode().toString());
 
 	// Some basic data validation to produce better error messages in cases of incorrect install
 	auto testFile = [](std::string filename, std::string message) -> bool
@@ -340,14 +344,14 @@ std::unique_ptr<CMap> MainWindow::openMapInternal(const QString & filenameSelect
 	if(auto header = mapService.loadMapHeader(resId))
 	{
 		auto missingMods = CMapService::verifyMapHeaderMods(*header);
-		ModIncompatibility::ModListWithVersion modList;
+		ModIncompatibility::ModList modList;
 		for(const auto & m : missingMods)
-			modList.push_back({m.second.name, m.second.version.toString()});
+			modList.push_back(m.second.name);
 		
 		if(!modList.empty())
 			throw ModIncompatibility(modList);
 		
-		return mapService.loadMap(resId);
+		return mapService.loadMap(resId, nullptr);
 	}
 	else
 		throw std::runtime_error("Corrupted map");
@@ -382,7 +386,7 @@ void MainWindow::on_actionOpen_triggered()
 		return;
 	
 	auto filenameSelect = QFileDialog::getOpenFileName(this, tr("Open map"),
-		QString::fromStdString(VCMIDirs::get().userCachePath().make_preferred().string()),
+		QString::fromStdString(VCMIDirs::get().userDataPath().make_preferred().string()),
 		tr("All supported maps (*.vmap *.h3m);;VCMI maps(*.vmap);;HoMM3 maps(*.h3m)"));
 	if(filenameSelect.isEmpty())
 		return;
@@ -439,11 +443,13 @@ void MainWindow::on_actionSave_as_triggered()
 	if(filenameSelect.isNull())
 		return;
 
-	if(filenameSelect == filename)
-		return;
+	QFileInfo fileInfo(filenameSelect);
+	lastSavingDir = fileInfo.dir().path();
+
+	if(fileInfo.suffix().toLower() != "vmap")
+		filenameSelect += ".vmap";
 
 	filename = filenameSelect;
-	lastSavingDir = filenameSelect.remove(QUrl(filenameSelect).fileName());
 
 	saveMap();
 }
@@ -538,14 +544,15 @@ void MainWindow::addGroupIntoCatalog(const std::string & groupName, bool useCust
 			auto picture = animation.getImage(0);
 			if(picture && picture->width() && picture->height())
 			{
-				qreal xscale = qreal(128) / qreal(picture->width()), yscale = qreal(128) / qreal(picture->height());
+				qreal xscale = static_cast<qreal>(128) / static_cast<qreal>(picture->width());
+				qreal yscale = static_cast<qreal>(128) / static_cast<qreal>(picture->height());
 				qreal scale = std::min(xscale, yscale);
 				painter.scale(scale, scale);
 				painter.drawImage(QPoint(0, 0), *picture);
 			}
 			
 			//create object to extract name
-			std::unique_ptr<CGObjectInstance> temporaryObj(factory->create(templ));
+			std::unique_ptr<CGObjectInstance> temporaryObj(factory->create(nullptr, templ));
 			QString translated = useCustomName ? QString::fromStdString(temporaryObj->getObjectName().c_str()) : subGroupName;
 			itemType->setText(translated);
 			
@@ -585,7 +592,7 @@ void MainWindow::loadObjectsTree()
 	//adding terrains
 	for(auto & terrain : VLC->terrainTypeHandler->objects)
 	{
-		QPushButton *b = new QPushButton(QString::fromStdString(terrain->getNameTranslated()));
+		auto *b = new QPushButton(QString::fromStdString(terrain->getNameTranslated()));
 		ui->terrainLayout->addWidget(b);
 		connect(b, &QPushButton::clicked, this, [this, terrain]{ terrainButtonClicked(terrain->getId()); });
 
@@ -600,7 +607,7 @@ void MainWindow::loadObjectsTree()
 	//adding roads
 	for(auto & road : VLC->roadTypeHandler->objects)
 	{
-		QPushButton *b = new QPushButton(QString::fromStdString(road->getNameTranslated()));
+		auto *b = new QPushButton(QString::fromStdString(road->getNameTranslated()));
 		ui->roadLayout->addWidget(b);
 		connect(b, &QPushButton::clicked, this, [this, road]{ roadOrRiverButtonClicked(road->getIndex(), true); });
 	}
@@ -609,7 +616,7 @@ void MainWindow::loadObjectsTree()
 	//adding rivers
 	for(auto & river : VLC->riverTypeHandler->objects)
 	{
-		QPushButton *b = new QPushButton(QString::fromStdString(river->getNameTranslated()));
+		auto *b = new QPushButton(QString::fromStdString(river->getNameTranslated()));
 		ui->riverLayout->addWidget(b);
 		connect(b, &QPushButton::clicked, this, [this, river]{ roadOrRiverButtonClicked(river->getIndex(), false); });
 	}
@@ -1170,7 +1177,7 @@ void MainWindow::on_actionTranslations_triggered()
 void MainWindow::on_actionh3m_converter_triggered()
 {
 	auto mapFiles = QFileDialog::getOpenFileNames(this, tr("Select maps to convert"),
-		QString::fromStdString(VCMIDirs::get().userCachePath().make_preferred().string()),
+		QString::fromStdString(VCMIDirs::get().userDataPath().make_preferred().string()),
 		tr("HoMM3 maps(*.h3m)"));
 	if(mapFiles.empty())
 		return;
