@@ -27,6 +27,7 @@ class MainWindow : public QMainWindow
 
 	const QString mainWindowSizeSetting = "MainWindow/Size";
 	const QString mainWindowPositionSetting = "MainWindow/Position";
+	const QString lastDirectorySetting = "MainWindow/Directory";
 
 #ifdef ENABLE_QT_TRANSLATIONS
 	QTranslator translator;
@@ -174,7 +175,8 @@ private:
 	ObjectBrowserProxyModel * objectBrowser = nullptr;
 	QGraphicsScene * scenePreview;
 	
-	QString filename, lastSavingDir;
+	QString filename;
+	QString lastSavingDir;
 	bool unsaved = false;
 
 	QStandardItemModel objectsModel;
