@@ -9,25 +9,21 @@
  */
 #pragma once
 
-#include "../lib/serializer/Connection.h"
+#include "../lib/network/NetworkInterface.h"
 #include "../lib/StartInfo.h"
 
-#include <boost/program_options.hpp>
-
-#if defined(VCMI_ANDROID) && !defined(SINGLE_PROCESS_APP)
-#define VCMI_ANDROID_DUAL_PROCESS 1
-#endif
-
 VCMI_LIB_NAMESPACE_BEGIN
 
 class CMapInfo;
 
 struct CPackForLobby;
 
+class CConnection;
 struct StartInfo;
 struct LobbyInfo;
 struct PlayerSettings;
 class PlayerColor;
+class MetaString;
 
 template<typename T> class CApplier;
 
@@ -36,72 +32,83 @@ VCMI_LIB_NAMESPACE_END
 class CGameHandler;
 class CBaseForServerApply;
 class CBaseForGHApply;
+class GlobalLobbyProcessor;
 
 enum class EServerState : ui8
 {
 	LOBBY,
-	GAMEPLAY_STARTING,
 	GAMEPLAY,
-	GAMEPLAY_ENDED,
 	SHUTDOWN
 };
 
-class CVCMIServer : public LobbyInfo
+class CVCMIServer : public LobbyInfo, public INetworkServerListener, public INetworkTimerListener
 {
-	std::atomic<bool> restartGameplay; // FIXME: this is just a hack
-	std::shared_ptr<boost::asio::io_service> io;
-	std::shared_ptr<TAcceptor> acceptor;
-	std::shared_ptr<TSocket> upcomingConnection;
-	std::list<std::unique_ptr<CPackForLobby>> announceQueue;
-	boost::recursive_mutex mx;
+	/// Network server instance that receives and processes incoming connections on active socket
+	std::unique_ptr<INetworkServer> networkServer;
+	std::unique_ptr<GlobalLobbyProcessor> lobbyProcessor;
+
+	std::chrono::steady_clock::time_point gameplayStartTime;
+	std::chrono::steady_clock::time_point lastTimerUpdateTime;
+
+	std::unique_ptr<INetworkHandler> networkHandler;
+
 	std::shared_ptr<CApplier<CBaseForServerApply>> applier;
-	std::unique_ptr<boost::thread> announceLobbyThread, remoteConnectionsThread;
-	std::atomic<EServerState> state;
+	EServerState state = EServerState::LOBBY;
+
+	std::shared_ptr<CConnection> findConnection(const std::shared_ptr<INetworkConnection> &);
+
+	int currentClientId;
+	ui8 currentPlayerId;
+	uint16_t port;
+	bool runByClient;
 
 public:
+	/// List of all active connections
+	std::vector<std::shared_ptr<CConnection>> activeConnections;
+
+	// INetworkListener impl
+	void onDisconnected(const std::shared_ptr<INetworkConnection> & connection, const std::string & errorMessage) override;
+	void onPacketReceived(const std::shared_ptr<INetworkConnection> & connection, const std::vector<std::byte> & message) override;
+	void onNewConnection(const std::shared_ptr<INetworkConnection> &) override;
+	void onTimer() override;
+
 	std::shared_ptr<CGameHandler> gh;
-	ui16 port;
 
-	boost::program_options::variables_map cmdLineOptions;
-	std::set<std::shared_ptr<CConnection>> connections;
-	std::set<std::shared_ptr<CConnection>> remoteConnections;
-	std::set<std::shared_ptr<CConnection>> hangingConnections; //keep connections of players disconnected during the game
-	
-	std::atomic<int> currentClientId;
-	std::atomic<ui8> currentPlayerId;
-	std::shared_ptr<CConnection> hostClient;
-
-	CVCMIServer(boost::program_options::variables_map & opts);
+	CVCMIServer(uint16_t port, bool connectToLobby, bool runByClient);
 	~CVCMIServer();
+
 	void run();
+
+	bool wasStartedByClient() const;
 	bool prepareToStartGame();
 	void prepareToRestart();
-	void startGameImmidiately();
+	void startGameImmediately();
+	void startAcceptingIncomingConnections();
 
-	void establishRemoteConnections();
-	void connectToRemote();
-	void startAsyncAccept();
-	void connectionAccepted(const boost::system::error_code & ec);
 	void threadHandleClient(std::shared_ptr<CConnection> c);
-	void threadAnnounceLobby();
-	void handleReceivedPack(std::unique_ptr<CPackForLobby> pack);
 
 	void announcePack(std::unique_ptr<CPackForLobby> pack);
 	bool passHost(int toConnectionId);
 
+	void announceTxt(MetaString txt, const std::string & playerName = "system");
 	void announceTxt(const std::string & txt, const std::string & playerName = "system");
-	void announceMessage(const std::string & txt);
-	void addToAnnounceQueue(std::unique_ptr<CPackForLobby> pack);
 
 	void setPlayerConnectedId(PlayerSettings & pset, ui8 player) const;
 	void updateStartInfoOnMapChange(std::shared_ptr<CMapInfo> mapInfo, std::shared_ptr<CMapGenOptions> mapGenOpt = {});
 
-	void clientConnected(std::shared_ptr<CConnection> c, std::vector<std::string> & names, std::string uuid, StartInfo::EMode mode);
+	void clientConnected(std::shared_ptr<CConnection> c, std::vector<std::string> & names, const std::string & uuid, EStartMode mode);
 	void clientDisconnected(std::shared_ptr<CConnection> c);
 	void reconnectPlayer(int connId);
 
+	void announceMessage(MetaString txt);
+	void announceMessage(const std::string & txt);
+
+	void handleReceivedPack(std::unique_ptr<CPackForLobby> pack);
+
 	void updateAndPropagateLobbyState();
 
+	INetworkHandler & getNetworkHandler();
+
 	void setState(EServerState value);
 	EServerState getState() const;
 
@@ -124,12 +131,5 @@ public:
 
 	ui8 getIdOfFirstUnallocatedPlayer() const;
 
-#if VCMI_ANDROID_DUAL_PROCESS
-	static void create();
-#elif defined(SINGLE_PROCESS_APP)
-	static void create(boost::condition_variable * cond, const std::vector<std::string> & args);
-# ifdef VCMI_ANDROID
-	static void reuseClientJNIEnv(void * jniEnv);
-# endif // VCMI_ANDROID
-#endif // VCMI_ANDROID_DUAL_PROCESS
+	void multiplayerWelcomeMessage();
 };
