@@ -19,7 +19,7 @@ using namespace ::testing;
 class CHeroClassTest : public Test
 {
 public:
-	MOCK_METHOD3(registarCb, void(int32_t, const std::string &, const std::string &));
+	MOCK_METHOD4(registarCb, void(int32_t, int32_t, const std::string &, const std::string &));
 
 protected:
 	std::shared_ptr<CHeroClass> subject;
@@ -32,7 +32,10 @@ protected:
 
 TEST_F(CHeroClassTest, RegistersIconsDoesNothing)
 {
-	auto cb = std::bind(&CHeroClassTest::registarCb, this, _1, _2, _3);
+	auto cb = [this](auto && PH1, auto && PH2, auto && PH3, auto && PH4) 
+	{
+		registarCb(std::forward<decltype(PH1)>(PH1), std::forward<decltype(PH2)>(PH2), std::forward<decltype(PH3)>(PH3), std::forward<decltype(PH4)>(PH4));
+	};
 	subject->registerIcons(cb);
 }
 
