@@ -14,6 +14,7 @@
 #include <vstd/RNG.h>
 
 #include "../../../lib/battle/CUnitState.h"
+#include "../../../lib/json/JsonNode.h"
 
 #include "mock/mock_UnitEnvironment.h"
 
@@ -84,7 +85,7 @@ protected:
 	}
 };
 
-TEST_F(DamageApplyTest, DoesDamageToAliveUnit)
+TEST_F(DamageApplyTest, DISABLED_DoesDamageToAliveUnit)
 {
 	EffectFixture::setupEffect(JsonNode());
 	using namespace ::battle;
@@ -104,7 +105,7 @@ TEST_F(DamageApplyTest, DoesDamageToAliveUnit)
 
 	unitsFake.setDefaultBonusExpectations();
 
-	std::shared_ptr<CUnitState> targetUnitState = std::make_shared<CUnitStateDetached>(&targetUnit, &targetUnit);
+	auto targetUnitState = std::make_shared<CUnitStateDetached>(&targetUnit, &targetUnit);
 	targetUnitState->localInit(&unitEnvironmentMock);
 	EXPECT_CALL(targetUnit, acquireState()).WillOnce(Return(targetUnitState));
 	EXPECT_CALL(*battleFake, setUnitState(Eq(unitId),_, Lt(0))).Times(1);
@@ -121,7 +122,7 @@ TEST_F(DamageApplyTest, DoesDamageToAliveUnit)
 	EXPECT_EQ(targetUnitState->getCount(), unitAmount - 1);
 }
 
-TEST_F(DamageApplyTest, IgnoresDeadUnit)
+TEST_F(DamageApplyTest, DISABLED_IgnoresDeadUnit)
 {
 	EffectFixture::setupEffect(JsonNode());
 	using namespace ::battle;
@@ -141,12 +142,12 @@ TEST_F(DamageApplyTest, IgnoresDeadUnit)
 	subject->apply(&serverMock, &mechanicsMock, target);
 }
 
-TEST_F(DamageApplyTest, DoesDamageByPercent)
+TEST_F(DamageApplyTest, DISABLED_DoesDamageByPercent)
 {
 	using namespace ::battle;
 
 	{
-		JsonNode config(JsonNode::JsonType::DATA_STRUCT);
+		JsonNode config;
 		config["killByPercentage"].Bool() = true;
 		EffectFixture::setupEffect(config);
 	}
@@ -168,7 +169,7 @@ TEST_F(DamageApplyTest, DoesDamageByPercent)
 
 	unitsFake.setDefaultBonusExpectations();
 
-	std::shared_ptr<CUnitState> targetUnitState = std::make_shared<CUnitStateDetached>(&targetUnit, &targetUnit);
+	auto targetUnitState = std::make_shared<CUnitStateDetached>(&targetUnit, &targetUnit);
 	targetUnitState->localInit(&unitEnvironmentMock);
 	EXPECT_CALL(targetUnit, acquireState()).WillOnce(Return(targetUnitState));
 
@@ -186,12 +187,12 @@ TEST_F(DamageApplyTest, DoesDamageByPercent)
 	EXPECT_EQ(targetUnitState->getCount(), unitAmount - (unitAmount * effectValue / 100));
 }
 
-TEST_F(DamageApplyTest, DoesDamageByCount)
+TEST_F(DamageApplyTest, DISABLED_DoesDamageByCount)
 {
 	using namespace ::battle;
 
 	{
-		JsonNode config(JsonNode::JsonType::DATA_STRUCT);
+		JsonNode config;
 		config["killByCount"].Bool() = true;
 		EffectFixture::setupEffect(config);
 	}
@@ -212,7 +213,7 @@ TEST_F(DamageApplyTest, DoesDamageByCount)
 
 	unitsFake.setDefaultBonusExpectations();
 
-	std::shared_ptr<CUnitState> targetUnitState = std::make_shared<CUnitStateDetached>(&targetUnit, &targetUnit);
+	auto targetUnitState = std::make_shared<CUnitStateDetached>(&targetUnit, &targetUnit);
 	targetUnitState->localInit(&unitEnvironmentMock);
 	EXPECT_CALL(targetUnit, acquireState()).WillOnce(Return(targetUnitState));
 
